@@ -5,25 +5,23 @@ use core::arch::x86_64::*;
 
 use core::mem::MaybeUninit;
 
-use crate::{
-    const_m128,
-    core::{
-        storage::{Align16, Columns2, Columns3, Columns4, XY, XYZ},
-        traits::{
-            matrix::{
-                FloatMatrix2x2, FloatMatrix3x3, FloatMatrix4x4, Matrix, Matrix2x2, Matrix3x3,
-                Matrix4x4, MatrixConst,
-            },
-            projection::ProjectionMatrix,
-            vector::{FloatVector4, Vector, Vector4, Vector4Const},
+use crate::core::{
+    storage::{Align16, Columns2, Columns3, Columns4, XY, XYZ},
+    traits::{
+        matrix::{
+            FloatMatrix2x2, FloatMatrix3x3, FloatMatrix4x4, Matrix, Matrix2x2, Matrix3x3,
+            Matrix4x4, MatrixConst,
         },
+        projection::ProjectionMatrix,
+        scalar::NanConstEx,
+        vector::{FloatVector4, Vector, Vector4, Vector4Const},
     },
 };
 
 // __m128 as a Matrix2x2
 impl MatrixConst for __m128 {
-    const ZERO: __m128 = const_m128!([0.0, 0.0, 0.0, 0.0]);
-    const IDENTITY: __m128 = const_m128!([1.0, 0.0, 0.0, 1.0]);
+    const ZERO: __m128 = const_f32x4!([0.0, 0.0, 0.0, 0.0]);
+    const IDENTITY: __m128 = const_f32x4!([1.0, 0.0, 0.0, 1.0]);
 }
 
 impl Matrix<f32> for __m128 {}
@@ -41,12 +39,12 @@ impl Matrix2x2<f32, XY<f32>> for __m128 {
 
     #[inline(always)]
     fn x_axis(&self) -> &XY<f32> {
-        unsafe { &(*(self as *const Self as *const Columns2<XY<f32>>)).x_axis }
+        unsafe { &(*(self as *const Self).cast::<Columns2<XY<f32>>>()).x_axis }
     }
 
     #[inline(always)]
     fn y_axis(&self) -> &XY<f32> {
-        unsafe { &(*(self as *const Self as *const Columns2<XY<f32>>)).y_axis }
+        unsafe { &(*(self as *const Self).cast::<Columns2<XY<f32>>>()).y_axis }
     }
 
     #[inline]
@@ -75,7 +73,7 @@ impl Matrix2x2<f32, XY<f32>> for __m128 {
             let cydyaxbx = _mm_shuffle_ps(axbxcydy, axbxcydy, 0b01_00_11_10);
             let result = _mm_add_ps(axbxcydy, cydyaxbx);
             let mut out: MaybeUninit<Align16<XY<f32>>> = MaybeUninit::uninit();
-            _mm_store_ps(out.as_mut_ptr() as *mut f32, result);
+            _mm_store_ps(out.as_mut_ptr().cast(), result);
             out.assume_init().0
         }
     }
@@ -122,7 +120,7 @@ impl FloatMatrix2x2<f32, XY<f32>> for __m128 {
     #[inline]
     fn inverse(&self) -> Self {
         unsafe {
-            const SIGN: __m128 = const_m128!([1.0, -1.0, -1.0, 1.0]);
+            const SIGN: __m128 = const_f32x4!([1.0, -1.0, -1.0, 1.0]);
             let abcd = *self;
             let dcba = _mm_shuffle_ps(abcd, abcd, 0b00_01_10_11);
             let prod = _mm_mul_ps(abcd, dcba);
@@ -149,6 +147,14 @@ impl MatrixConst for Columns3<__m128> {
     };
 }
 
+impl NanConstEx for Columns3<__m128> {
+    const NAN: Columns3<__m128> = Columns3 {
+        x_axis: __m128::NAN,
+        y_axis: __m128::NAN,
+        z_axis: __m128::NAN,
+    };
+}
+
 impl Matrix<f32> for Columns3<__m128> {}
 
 impl Matrix3x3<f32, __m128> for Columns3<__m128> {
@@ -223,6 +229,15 @@ impl MatrixConst for Columns4<__m128> {
     };
 }
 
+impl NanConstEx for Columns4<__m128> {
+    const NAN: Columns4<__m128> = Columns4 {
+        x_axis: __m128::NAN,
+        y_axis: __m128::NAN,
+        z_axis: __m128::NAN,
+        w_axis: __m128::NAN,
+    };
+}
+
 impl Matrix<f32> for Columns4<__m128> {}
 
 impl Matrix4x4<f32, __m128> for Columns4<__m128> {
