@@ -5,7 +5,8 @@ use crate::core::{
 use crate::{DMat2, DMat4, DQuat, DVec2, DVec3, EulerRot, Mat2, Mat4, Quat, Vec2, Vec3, Vec3A};
 #[cfg(not(target_arch = "spirv"))]
 use core::fmt;
-use core::ops::{Add, Deref, DerefMut, Mul, Sub};
+use core::iter::{Product, Sum};
+use core::ops::{Add, AddAssign, Deref, DerefMut, Mul, MulAssign, Sub, SubAssign};
 
 #[cfg(all(
     target_arch = "x86",
@@ -20,8 +21,8 @@ use core::arch::x86::*;
 ))]
 use core::arch::x86_64::*;
 
-#[cfg(feature = "std")]
-use std::iter::{Product, Sum};
+#[cfg(target_feature = "simd128")]
+use core::arch::wasm32::v128;
 
 macro_rules! define_mat3_struct {
     ($mat3:ident, $inner:ident) => {
@@ -29,7 +30,8 @@ macro_rules! define_mat3_struct {
         ///
         /// This 3x3 matrix type features convenience methods for creating and using linear and
         /// affine transformations. If you are primarily dealing with 2D affine transformations the
-        /// [`Affine2`] type is much faster and more space efficient than using a 3x3 matrix.
+        /// [`Affine2`](crate::Affine2) type is much faster and more space efficient than using a
+        /// 3x3 matrix.
         ///
         /// Linear transformations including 3D rotation and scale can be created using methods
         /// such as [`Self::from_diagonal()`], [`Self::from_quat()`], [`Self::from_axis_angle()`],
@@ -55,7 +57,7 @@ macro_rules! define_mat3_struct {
 }
 
 macro_rules! impl_mat3_methods {
-    ($t:ty, $vec3:ident, $vec3a:ident, $vec2:ident, $quat:ident, $mat2:ident, $inner:ident) => {
+    ($t:ty, $vec3:ident, $vec3a:ident, $vec2:ident, $quat:ident, $mat2:ident, $mat4:ident, $inner:ident) => {
         /// A 3x3 matrix with all elements set to `0.0`.
         pub const ZERO: Self = Self($inner::ZERO);
 
@@ -63,6 +65,9 @@ macro_rules! impl_mat3_methods {
         /// elements are `0`.
         pub const IDENTITY: Self = Self($inner::IDENTITY);
 
+        /// All NAN:s.
+        pub const NAN: Self = Self(<$inner as crate::core::traits::scalar::NanConstEx>::NAN);
+
         /// Creates a 3x3 matrix from three column vectors.
         #[inline(always)]
         pub fn from_cols(x_axis: $vec3a, y_axis: $vec3a, z_axis: $vec3a) -> Self {
@@ -101,14 +106,27 @@ macro_rules! impl_mat3_methods {
 
         /// Creates a 3x3 matrix with its diagonal set to `diagonal` and all other entries set to 0.
         /// The resulting matrix is a 3D scale transfom.
-        #[cfg_attr(docsrs, doc(alias = "scale"))]
+        #[doc(alias = "scale")]
         #[inline(always)]
         pub fn from_diagonal(diagonal: $vec3) -> Self {
             Self($inner::from_diagonal(diagonal.0))
         }
 
-        #[inline(always)]
+        /// Creates a 3x3 matrix from a 4x4 matrix, discarding the 3rd row and column.
+        pub fn from_mat4(m: $mat4) -> Self {
+            Self::from_cols(
+                $vec3a(m.x_axis.0.into()),
+                $vec3a(m.y_axis.0.into()),
+                $vec3a(m.z_axis.0.into()),
+            )
+        }
+
         /// Creates a 3D rotation matrix from the given quaternion.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `rotation` is not normalized when `glam_assert` is enabled.
+        #[inline(always)]
         pub fn from_quat(rotation: $quat) -> Self {
             // TODO: SIMD?
             Self($inner::from_quaternion(rotation.0.into()))
@@ -116,20 +134,15 @@ macro_rules! impl_mat3_methods {
 
         /// Creates a 3D rotation matrix from a normalized rotation `axis` and `angle` (in
         /// radians).
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `axis` is not normalized when `glam_assert` is enabled.
         #[inline(always)]
         pub fn from_axis_angle(axis: $vec3, angle: $t) -> Self {
             Self(FloatMatrix3x3::from_axis_angle(axis.0, angle))
         }
 
-        #[deprecated(
-            since = "0.15.0",
-            note = "Please use `from_euler(EulerRot::YXZ, yaw, pitch, roll)` instead"
-        )]
-        #[inline(always)]
-        pub fn from_rotation_ypr(yaw: $t, pitch: $t, roll: $t) -> Self {
-            Self::from_euler(EulerRot::YXZ, yaw, pitch, roll)
-        }
-
         #[inline(always)]
         /// Creates a 3D rotation matrix from the given euler rotation sequence and the angles (in
         /// radians).
@@ -193,11 +206,24 @@ macro_rules! impl_mat3_methods {
         ///
         /// The resulting matrix can be used to transform 2D points and vectors. See
         /// [`Self::transform_point2()`] and [`Self::transform_vector2()`].
+        ///
+        /// # Panics
+        ///
+        /// Will panic if all elements of `scale` are zero when `glam_assert` is enabled.
         #[inline(always)]
         pub fn from_scale(scale: $vec2) -> Self {
             Self(Matrix3x3::from_scale(scale.0))
         }
 
+        /// Creates an affine transformation matrix from the given 2x2 matrix.
+        ///
+        /// The resulting matrix can be used to transform 2D points and vectors. See
+        /// [`Self::transform_point2()`] and [`Self::transform_vector2()`].
+        #[inline(always)]
+        pub fn from_mat2(m: $mat2) -> Self {
+            Self::from_cols((m.x_axis, 0.0).into(), (m.y_axis, 0.0).into(), $vec3a::Z)
+        }
+
         /// Creates a 3x3 matrix from the first 9 values in `slice`.
         ///
         /// # Panics
@@ -229,10 +255,22 @@ macro_rules! impl_mat3_methods {
                 0 => self.x_axis,
                 1 => self.y_axis,
                 2 => self.z_axis,
-                _ => panic!(
-                    "index out of bounds: the len is 3 but the index is {}",
-                    index
-                ),
+                _ => panic!("index out of bounds"),
+            }
+        }
+
+        /// Returns a mutable reference to the matrix column for the given `index`.
+        ///
+        /// # Panics
+        ///
+        /// Panics if `index` is greater than 2.
+        #[inline]
+        pub fn col_mut(&mut self, index: usize) -> &mut $vec3a {
+            match index {
+                0 => &mut self.x_axis,
+                1 => &mut self.y_axis,
+                2 => &mut self.z_axis,
+                _ => panic!("index out of bounds"),
             }
         }
 
@@ -247,26 +285,10 @@ macro_rules! impl_mat3_methods {
                 0 => $vec3a::new(self.x_axis.x, self.y_axis.x, self.z_axis.x),
                 1 => $vec3a::new(self.x_axis.y, self.y_axis.y, self.z_axis.y),
                 2 => $vec3a::new(self.x_axis.z, self.y_axis.z, self.z_axis.z),
-                _ => panic!(
-                    "index out of bounds: the len is 3 but the index is {}",
-                    index
-                ),
+                _ => panic!("index out of bounds"),
             }
         }
 
-        // #[inline]
-        // pub(crate) fn col_mut(&mut self, index: usize) -> &mut $vec3 {
-        //     match index {
-        //         0 => &mut self.x_axis,
-        //         1 => &mut self.y_axis,
-        //         2 => &mut self.z_axis,
-        //         _ => panic!(
-        //             "index out of bounds: the len is 3 but the index is {}",
-        //             index
-        //         ),
-        //     }
-        // }
-
         /// Returns `true` if, and only if, all elements are finite.
         /// If any element is either `NaN`, positive or negative infinity, this will return `false`.
         #[inline]
@@ -281,6 +303,7 @@ macro_rules! impl_mat3_methods {
         }
 
         /// Returns the transpose of `self`.
+        #[must_use]
         #[inline(always)]
         pub fn transpose(&self) -> Self {
             Self(self.0.transpose())
@@ -295,6 +318,11 @@ macro_rules! impl_mat3_methods {
         /// Returns the inverse of `self`.
         ///
         /// If the matrix is not invertible the returned matrix will be invalid.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if the determinant of `self` is zero when `glam_assert` is enabled.
+        #[must_use]
         #[inline(always)]
         pub fn inverse(&self) -> Self {
             Self(self.0.inverse())
@@ -337,7 +365,8 @@ macro_rules! impl_mat3_methods {
         /// This method assumes that `self` contains a valid affine transform.
         #[inline(always)]
         pub fn transform_point2(&self, other: $vec2) -> $vec2 {
-            $mat2::from_cols(self.x_axis.into(), self.y_axis.into()) * other + self.z_axis.into()
+            $mat2::from_cols($vec2(self.x_axis.0.into()), $vec2(self.y_axis.0.into())) * other
+                + $vec2(self.z_axis.0.into())
         }
 
         /// Rotates the given 2D vector.
@@ -347,7 +376,7 @@ macro_rules! impl_mat3_methods {
         /// This method assumes that `self` contains a valid affine transform.
         #[inline(always)]
         pub fn transform_vector2(&self, other: $vec2) -> $vec2 {
-            $mat2::from_cols(self.x_axis.into(), self.y_axis.into()) * other
+            $mat2::from_cols($vec2(self.x_axis.0.into()), $vec2(self.y_axis.0.into())) * other
         }
 
         /// Returns true if the absolute difference of all elements between `self` and `other`
@@ -367,19 +396,14 @@ macro_rules! impl_mat3_methods {
 }
 
 macro_rules! impl_mat3_traits {
-    ($t:ty, $new:ident, $mat3:ident, $mat4:ident, $vec3:ident, $vec3a:ident) => {
+    ($t:ty, $new:ident, $mat3:ident, $vec3:ident, $vec3a:ident) => {
         /// Creates a 3x3 matrix from three column vectors.
         #[inline(always)]
         pub fn $new(x_axis: $vec3a, y_axis: $vec3a, z_axis: $vec3a) -> $mat3 {
             $mat3::from_cols(x_axis, y_axis, z_axis)
         }
 
-        impl Default for $mat3 {
-            #[inline(always)]
-            fn default() -> Self {
-                Self::IDENTITY
-            }
-        }
+        impl_matn_common_traits!($t, $mat3, $vec3a);
 
         impl PartialEq for $mat3 {
             #[inline]
@@ -407,14 +431,14 @@ macro_rules! impl_mat3_traits {
 
         #[cfg(not(target_arch = "spirv"))]
         impl fmt::Display for $mat3 {
-            fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
                 write!(f, "[{}, {}, {}]", self.x_axis, self.y_axis, self.z_axis)
             }
         }
 
         #[cfg(not(target_arch = "spirv"))]
         impl fmt::Debug for $mat3 {
-            fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+            fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
                 fmt.debug_struct("$mat3")
                     .field("x_axis", &self.x_axis)
                     .field("y_axis", &self.y_axis)
@@ -422,86 +446,12 @@ macro_rules! impl_mat3_traits {
                     .finish()
             }
         }
-
-        impl Add<$mat3> for $mat3 {
-            type Output = Self;
-            #[inline(always)]
-            fn add(self, other: Self) -> Self {
-                self.add_mat3(&other)
-            }
-        }
-
-        impl Sub<$mat3> for $mat3 {
-            type Output = Self;
-            #[inline(always)]
-            fn sub(self, other: Self) -> Self {
-                self.sub_mat3(&other)
-            }
-        }
-
-        impl Mul<$mat3> for $mat3 {
-            type Output = Self;
-            #[inline(always)]
-            fn mul(self, other: Self) -> Self {
-                self.mul_mat3(&other)
-            }
-        }
-
-        impl Mul<$vec3> for $mat3 {
-            type Output = $vec3;
-            #[inline(always)]
-            fn mul(self, other: $vec3) -> $vec3 {
-                self.mul_vec3(other)
-            }
-        }
-
-        impl Mul<$mat3> for $t {
-            type Output = $mat3;
-            #[inline(always)]
-            fn mul(self, other: $mat3) -> $mat3 {
-                other.mul_scalar(self)
-            }
-        }
-
-        impl Mul<$t> for $mat3 {
-            type Output = Self;
-            #[inline(always)]
-            fn mul(self, other: $t) -> Self {
-                self.mul_scalar(other)
-            }
-        }
-
-        impl From<$mat4> for $mat3 {
-            /// Creates a 3x3 matrix from the top left submatrix of the given 4x4 matrix.
-            fn from(m: $mat4) -> $mat3 {
-                $mat3::from_cols(m.x_axis.into(), m.y_axis.into(), m.z_axis.into())
-            }
-        }
-
-        #[cfg(feature = "std")]
-        impl<'a> Sum<&'a Self> for $mat3 {
-            fn sum<I>(iter: I) -> Self
-            where
-                I: Iterator<Item = &'a Self>,
-            {
-                iter.fold($mat3::ZERO, |a, &b| Self::add(a, b))
-            }
-        }
-
-        #[cfg(feature = "std")]
-        impl<'a> Product<&'a Self> for $mat3 {
-            fn product<I>(iter: I) -> Self
-            where
-                I: Iterator<Item = &'a Self>,
-            {
-                iter.fold($mat3::IDENTITY, |a, &b| Self::mul(a, b))
-            }
-        }
     };
 }
 
 macro_rules! impl_mat3_traits_unsafe {
     ($t:ty, $mat3:ident) => {
+        #[cfg(not(target_arch = "spriv"))]
         impl AsRef<[$t; 9]> for $mat3 {
             #[inline(always)]
             fn as_ref(&self) -> &[$t; 9] {
@@ -509,6 +459,7 @@ macro_rules! impl_mat3_traits_unsafe {
             }
         }
 
+        #[cfg(not(target_arch = "spriv"))]
         impl AsMut<[$t; 9]> for $mat3 {
             #[inline(always)]
             fn as_mut(&mut self) -> &mut [$t; 9] {
@@ -522,7 +473,7 @@ type InnerF32 = Columns3<XYZ<f32>>;
 define_mat3_struct!(Mat3, InnerF32);
 
 impl Mat3 {
-    impl_mat3_methods!(f32, Vec3, Vec3, Vec2, Quat, Mat2, InnerF32);
+    impl_mat3_methods!(f32, Vec3, Vec3, Vec2, Quat, Mat2, Mat4, InnerF32);
 
     /// Transforms a `Vec3A`.
     #[inline]
@@ -530,16 +481,22 @@ impl Mat3 {
         self.mul_vec3(other.into()).into()
     }
 
+    #[deprecated(since = "0.18.0", note = "please use `as_dmat3()` instead")]
     #[inline(always)]
     pub fn as_f64(&self) -> DMat3 {
+        self.as_dmat3()
+    }
+
+    #[inline(always)]
+    pub fn as_dmat3(&self) -> DMat3 {
         DMat3::from_cols(
-            self.x_axis.as_f64(),
-            self.y_axis.as_f64(),
-            self.z_axis.as_f64(),
+            self.x_axis.as_dvec3(),
+            self.y_axis.as_dvec3(),
+            self.z_axis.as_dvec3(),
         )
     }
 }
-impl_mat3_traits!(f32, mat3, Mat3, Mat4, Vec3, Vec3);
+impl_mat3_traits!(f32, mat3, Mat3, Vec3, Vec3);
 impl_mat3_traits_unsafe!(f32, Mat3);
 
 impl Mul<Vec3A> for Mat3 {
@@ -552,12 +509,19 @@ impl Mul<Vec3A> for Mat3 {
 
 #[cfg(all(target_feature = "sse2", not(feature = "scalar-math")))]
 type InnerF32A = Columns3<__m128>;
-#[cfg(any(not(target_feature = "sse2"), feature = "scalar-math"))]
+
+#[cfg(all(target_feature = "simd128", not(feature = "scalar-math")))]
+type InnerF32A = Columns3<v128>;
+
+#[cfg(any(
+    not(any(target_feature = "sse2", target_feature = "simd128")),
+    feature = "scalar-math"
+))]
 type InnerF32A = Columns3<crate::core::storage::XYZF32A16>;
 define_mat3_struct!(Mat3A, InnerF32A);
 
 impl Mat3A {
-    impl_mat3_methods!(f32, Vec3, Vec3A, Vec2, Quat, Mat2, InnerF32A);
+    impl_mat3_methods!(f32, Vec3, Vec3A, Vec2, Quat, Mat2, Mat4, InnerF32A);
 
     /// Transforms a `Vec3A`.
     #[inline]
@@ -565,22 +529,28 @@ impl Mat3A {
         Vec3A(self.0.mul_vector(other.0))
     }
 
+    #[deprecated(since = "0.18.0", note = "please use `as_dmat3()` instead")]
     #[inline(always)]
     pub fn as_f64(&self) -> DMat3 {
+        self.as_dmat3()
+    }
+
+    #[inline(always)]
+    pub fn as_dmat3(&self) -> DMat3 {
         DMat3::from_cols(
-            self.x_axis.as_f64(),
-            self.y_axis.as_f64(),
-            self.z_axis.as_f64(),
+            self.x_axis.as_dvec3(),
+            self.y_axis.as_dvec3(),
+            self.z_axis.as_dvec3(),
         )
     }
 }
-impl_mat3_traits!(f32, mat3a, Mat3A, Mat4, Vec3, Vec3A);
+impl_mat3_traits!(f32, mat3a, Mat3A, Vec3, Vec3A);
 
-impl Mul<Vec3A> for Mat3A {
-    type Output = Vec3A;
+impl Mul<Vec3> for Mat3A {
+    type Output = Vec3;
     #[inline(always)]
-    fn mul(self, other: Vec3A) -> Vec3A {
-        self.mul_vec3a(other)
+    fn mul(self, other: Vec3) -> Vec3 {
+        self.mul_vec3(other)
     }
 }
 
@@ -602,22 +572,31 @@ type InnerF64 = Columns3<XYZ<f64>>;
 define_mat3_struct!(DMat3, InnerF64);
 
 impl DMat3 {
-    impl_mat3_methods!(f64, DVec3, DVec3, DVec2, DQuat, DMat2, InnerF64);
+    impl_mat3_methods!(f64, DVec3, DVec3, DVec2, DQuat, DMat2, DMat4, InnerF64);
 
+    #[deprecated(since = "0.18.0", note = "please use `as_mat3()` instead")]
     #[inline(always)]
     pub fn as_f32(&self) -> Mat3 {
+        self.as_mat3()
+    }
+
+    #[inline(always)]
+    pub fn as_mat3(&self) -> Mat3 {
         Mat3::from_cols(
-            self.x_axis.as_f32(),
-            self.y_axis.as_f32(),
-            self.z_axis.as_f32(),
+            self.x_axis.as_vec3(),
+            self.y_axis.as_vec3(),
+            self.z_axis.as_vec3(),
         )
     }
 }
-impl_mat3_traits!(f64, dmat3, DMat3, DMat4, DVec3, DVec3);
+impl_mat3_traits!(f64, dmat3, DMat3, DVec3, DVec3);
 impl_mat3_traits_unsafe!(f64, DMat3);
 
 mod const_test_mat3 {
-    const_assert_eq!(4, core::mem::align_of::<super::Mat3>());
+    const_assert_eq!(
+        core::mem::align_of::<f32>(),
+        core::mem::align_of::<super::Mat3>()
+    );
     const_assert_eq!(36, core::mem::size_of::<super::Mat3>());
 }
 
@@ -627,6 +606,9 @@ mod const_test_mat3a {
 }
 
 mod const_test_dmat3 {
-    const_assert_eq!(8, core::mem::align_of::<super::DMat3>());
+    const_assert_eq!(
+        core::mem::align_of::<f64>(),
+        core::mem::align_of::<super::DMat3>()
+    );
     const_assert_eq!(72, core::mem::size_of::<super::DMat3>());
 }
