@@ -5,7 +5,7 @@ use crate::core::{
         projection::ProjectionMatrix,
     },
 };
-use crate::{DQuat, DVec3, DVec4, EulerRot, Quat, Vec3, Vec3A, Vec4};
+use crate::{DMat3, DQuat, DVec3, DVec4, EulerRot, Mat3, Quat, Vec3, Vec3A, Vec4};
 
 #[cfg(all(
     target_feature = "sse2",
@@ -20,12 +20,13 @@ use core::arch::x86::*;
 ))]
 use core::arch::x86_64::*;
 
+#[cfg(target_feature = "simd128")]
+use core::arch::wasm32::v128;
+
 #[cfg(not(target_arch = "spirv"))]
 use core::fmt;
-use core::ops::{Add, Deref, DerefMut, Mul, Sub};
-
-#[cfg(feature = "std")]
-use std::iter::{Product, Sum};
+use core::iter::{Product, Sum};
+use core::ops::{Add, AddAssign, Deref, DerefMut, Mul, MulAssign, Sub, SubAssign};
 
 //macro_rules! define_mat4_struct {
 //    ($mat4:ident, $inner:ident) => {
@@ -63,13 +64,16 @@ use std::iter::{Product, Sum};
 //}
 
 macro_rules! impl_mat4_methods {
-    ($t:ident, $vec4:ident, $vec3:ident, $quat:ident, $inner:ident) => {
+    ($t:ident, $vec4:ident, $vec3:ident, $mat3:ident, $quat:ident, $inner:ident) => {
         /// A 4x4 matrix with all elements set to `0.0`.
         pub const ZERO: Self = Self($inner::ZERO);
 
         /// A 4x4 identity matrix, where all diagonal elements are `1`, and all off-diagonal elements are `0`.
         pub const IDENTITY: Self = Self($inner::IDENTITY);
 
+        /// All NAN:s.
+        pub const NAN: Self = Self(<$inner as crate::core::traits::scalar::NanConstEx>::NAN);
+
         /// Creates a 4x4 matrix from four column vectors.
         #[inline(always)]
         pub fn from_cols(x_axis: $vec4, y_axis: $vec4, z_axis: $vec4, w_axis: $vec4) -> Self {
@@ -107,7 +111,7 @@ macro_rules! impl_mat4_methods {
         }
 
         /// Creates a 4x4 matrix with its diagonal set to `diagonal` and all other entries set to 0.
-        #[cfg_attr(docsrs, doc(alias = "scale"))]
+        #[doc(alias = "scale")]
         #[inline(always)]
         pub fn from_diagonal(diagonal: $vec4) -> Self {
             Self($inner::from_diagonal(diagonal.0.into()))
@@ -118,6 +122,10 @@ macro_rules! impl_mat4_methods {
         ///
         /// The resulting matrix can be used to transform 3D points and vectors. See
         /// [`Self::transform_point3()`] and [`Self::transform_vector3()`].
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `rotation` is not normalized when `glam_assert` is enabled.
         #[inline(always)]
         pub fn from_scale_rotation_translation(
             scale: $vec3,
@@ -135,6 +143,10 @@ macro_rules! impl_mat4_methods {
         ///
         /// The resulting matrix can be used to transform 3D points and vectors. See
         /// [`Self::transform_point3()`] and [`Self::transform_vector3()`].
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `rotation` is not normalized when `glam_assert` is enabled.
         #[inline(always)]
         pub fn from_rotation_translation(rotation: $quat, translation: $vec3) -> Self {
             Self($inner::from_quaternion_translation(
@@ -145,6 +157,11 @@ macro_rules! impl_mat4_methods {
 
         /// Extracts `scale`, `rotation` and `translation` from `self`. The input matrix is
         /// expected to be a 3D affine transformation matrix otherwise the output will be invalid.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if the determinant of `self` is zero or if the resulting scale vector
+        /// contains any zero elements when `glam_assert` is enabled.
         #[inline(always)]
         pub fn to_scale_rotation_translation(&self) -> ($vec3, $quat, $vec3) {
             let (scale, rotation, translation) = self.0.to_scale_quaternion_translation();
@@ -155,11 +172,30 @@ macro_rules! impl_mat4_methods {
         ///
         /// The resulting matrix can be used to transform 3D points and vectors. See
         /// [`Self::transform_point3()`] and [`Self::transform_vector3()`].
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `rotation` is not normalized when `glam_assert` is enabled.
         #[inline(always)]
         pub fn from_quat(rotation: $quat) -> Self {
             Self($inner::from_quaternion(rotation.0))
         }
 
+        /// Creates an affine transformation matrix from the given 3x3 linear transformation
+        /// matrix.
+        ///
+        /// The resulting matrix can be used to transform 3D points and vectors. See
+        /// [`Self::transform_point3()`] and [`Self::transform_vector3()`].
+        #[inline(always)]
+        pub fn from_mat3(m: $mat3) -> Self {
+            Self::from_cols(
+                (m.x_axis, 0.0).into(),
+                (m.y_axis, 0.0).into(),
+                (m.z_axis, 0.0).into(),
+                $vec4::W,
+            )
+        }
+
         /// Creates an affine transformation matrix from the given 3D `translation`.
         ///
         /// The resulting matrix can be used to transform 3D points and vectors. See
@@ -174,20 +210,15 @@ macro_rules! impl_mat4_methods {
         ///
         /// The resulting matrix can be used to transform 3D points and vectors. See
         /// [`Self::transform_point3()`] and [`Self::transform_vector3()`].
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `axis` is not normalized when `glam_assert` is enabled.
         #[inline(always)]
         pub fn from_axis_angle(axis: $vec3, angle: $t) -> Self {
             Self($inner::from_axis_angle(axis.0, angle))
         }
 
-        #[deprecated(
-            since = "0.15.0",
-            note = "Please use `from_euler(EulerRot::YXZ, yaw, pitch, roll)` instead"
-        )]
-        #[inline(always)]
-        pub fn from_rotation_ypr(yaw: $t, pitch: $t, roll: $t) -> Self {
-            Self::from_euler(EulerRot::YXZ, yaw, pitch, roll)
-        }
-
         #[inline(always)]
         /// Creates a affine transformation matrix containing a rotation from the given euler
         /// rotation sequence and angles (in radians).
@@ -233,6 +264,10 @@ macro_rules! impl_mat4_methods {
         ///
         /// The resulting matrix can be used to transform 3D points and vectors. See
         /// [`Self::transform_point3()`] and [`Self::transform_vector3()`].
+        ///
+        /// # Panics
+        ///
+        /// Will panic if all elements of `scale` are zero when `glam_assert` is enabled.
         #[inline(always)]
         pub fn from_scale(scale: $vec3) -> Self {
             Self($inner::from_scale(scale.0))
@@ -270,10 +305,23 @@ macro_rules! impl_mat4_methods {
                 1 => self.y_axis,
                 2 => self.z_axis,
                 3 => self.w_axis,
-                _ => panic!(
-                    "index out of bounds: the len is 4 but the index is {}",
-                    index
-                ),
+                _ => panic!("index out of bounds"),
+            }
+        }
+
+        /// Returns a mutable reference to the matrix column for the given `index`.
+        ///
+        /// # Panics
+        ///
+        /// Panics if `index` is greater than 3.
+        #[inline]
+        pub fn col_mut(&mut self, index: usize) -> &mut $vec4 {
+            match index {
+                0 => &mut self.x_axis,
+                1 => &mut self.y_axis,
+                2 => &mut self.z_axis,
+                3 => &mut self.w_axis,
+                _ => panic!("index out of bounds"),
             }
         }
 
@@ -289,27 +337,10 @@ macro_rules! impl_mat4_methods {
                 1 => $vec4::new(self.x_axis.y, self.y_axis.y, self.z_axis.y, self.w_axis.y),
                 2 => $vec4::new(self.x_axis.z, self.y_axis.z, self.z_axis.z, self.w_axis.z),
                 3 => $vec4::new(self.x_axis.w, self.y_axis.w, self.z_axis.w, self.w_axis.w),
-                _ => panic!(
-                    "index out of bounds: the len is 3 but the index is {}",
-                    index
-                ),
+                _ => panic!("index out of bounds"),
             }
         }
 
-        // #[inline]
-        // pub(crate) fn col_mut(&mut self, index: usize) -> &mut $vec4 {
-        //     match index {
-        //         0 => &mut self.x_axis,
-        //         1 => &mut self.y_axis,
-        //         2 => &mut self.z_axis,
-        //         3 => &mut self.w_axis,
-        //         _ => panic!(
-        //             "index out of bounds: the len is 4 but the index is {}",
-        //             index
-        //         ),
-        //     }
-        // }
-
         /// Returns `true` if, and only if, all elements are finite.
         /// If any element is either `NaN`, positive or negative infinity, this will return `false`.
         #[inline]
@@ -345,6 +376,10 @@ macro_rules! impl_mat4_methods {
         /// Returns the inverse of `self`.
         ///
         /// If the matrix is not invertible the returned matrix will be invalid.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if the determinant of `self` is zero when `glam_assert` is enabled.
         #[must_use]
         #[inline(always)]
         pub fn inverse(&self) -> Self {
@@ -354,6 +389,10 @@ macro_rules! impl_mat4_methods {
         /// Creates a left-handed view matrix using a camera position, an up direction, and a focal
         /// point.
         /// For a view coordinate system with `+X=right`, `+Y=up` and `+Z=forward`.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `up` is not normalized when `glam_assert` is enabled.
         #[inline(always)]
         pub fn look_at_lh(eye: $vec3, center: $vec3, up: $vec3) -> Self {
             Self($inner::look_at_lh(eye.0, center.0, up.0))
@@ -362,6 +401,10 @@ macro_rules! impl_mat4_methods {
         /// Creates a right-handed view matrix using a camera position, an up direction, and a focal
         /// point.
         /// For a view coordinate system with `+X=right`, `+Y=up` and `+Z=back`.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `up` is not normalized when `glam_assert` is enabled.
         #[inline(always)]
         pub fn look_at_rh(eye: $vec3, center: $vec3, up: $vec3) -> Self {
             Self($inner::look_at_rh(eye.0, center.0, up.0))
@@ -369,7 +412,7 @@ macro_rules! impl_mat4_methods {
 
         /// Creates a right-handed perspective projection matrix with [-1,1] depth range.
         /// This is the same as the OpenGL `gluPerspective` function.
-        /// See https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/gluPerspective.xml
+        /// See <https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/gluPerspective.xml>
         #[inline(always)]
         pub fn perspective_rh_gl(
             fov_y_radians: $t,
@@ -386,6 +429,11 @@ macro_rules! impl_mat4_methods {
         }
 
         /// Creates a left-handed perspective projection matrix with `[0,1]` depth range.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `z_near` or `z_far` are less than or equal to zero when `glam_assert` is
+        /// enabled.
         #[inline(always)]
         pub fn perspective_lh(fov_y_radians: $t, aspect_ratio: $t, z_near: $t, z_far: $t) -> Self {
             Self($inner::perspective_lh(
@@ -397,6 +445,11 @@ macro_rules! impl_mat4_methods {
         }
 
         /// Creates a right-handed perspective projection matrix with `[0,1]` depth range.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `z_near` or `z_far` are less than or equal to zero when `glam_assert` is
+        /// enabled.
         #[inline(always)]
         pub fn perspective_rh(fov_y_radians: $t, aspect_ratio: $t, z_near: $t, z_far: $t) -> Self {
             Self($inner::perspective_rh(
@@ -408,6 +461,10 @@ macro_rules! impl_mat4_methods {
         }
 
         /// Creates an infinite left-handed perspective projection matrix with `[0,1]` depth range.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `z_near` is less than or equal to zero when `glam_assert` is enabled.
         #[inline(always)]
         pub fn perspective_infinite_lh(fov_y_radians: $t, aspect_ratio: $t, z_near: $t) -> Self {
             Self($inner::perspective_infinite_lh(
@@ -418,6 +475,10 @@ macro_rules! impl_mat4_methods {
         }
 
         /// Creates an infinite left-handed perspective projection matrix with `[0,1]` depth range.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `z_near` is less than or equal to zero when `glam_assert` is enabled.
         #[inline(always)]
         pub fn perspective_infinite_reverse_lh(
             fov_y_radians: $t,
@@ -460,7 +521,7 @@ macro_rules! impl_mat4_methods {
         /// Creates a right-handed orthographic projection matrix with `[-1,1]` depth
         /// range.  This is the same as the OpenGL `glOrtho` function in OpenGL.
         /// See
-        /// https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/glOrtho.xml
+        /// <https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/glOrtho.xml>
         #[inline(always)]
         pub fn orthographic_rh_gl(
             left: $t,
@@ -504,7 +565,7 @@ macro_rules! impl_mat4_methods {
         /// Transforms a 4D vector.
         #[inline(always)]
         pub fn mul_vec4(&self, other: $vec4) -> $vec4 {
-            $vec4(self.0.mul_vector(&other.0))
+            $vec4(self.0.mul_vector(other.0))
         }
 
         /// Multiplies two 4x4 matrices.
@@ -550,6 +611,10 @@ macro_rules! impl_mat4_methods {
         /// This method assumes that `self` contains a valid affine transform. It does not perform
         /// a persective divide, if `self` contains a perspective transform, or if you are unsure,
         /// the [`Self::project_point3()`] method should be used instead.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if the 3rd row of `self` is not `(0, 0, 0, 1)` when `glam_assert` is enabled.
         #[inline]
         pub fn transform_point3(&self, other: $vec3) -> $vec3 {
             glam_assert!(self.row(3) == $vec4::W);
@@ -562,6 +627,10 @@ macro_rules! impl_mat4_methods {
         /// `0.0`.
         ///
         /// This method assumes that `self` contains a valid affine transform.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if the 3rd row of `self` is not `(0, 0, 0, 1)` when `glam_assert` is enabled.
         #[inline]
         pub fn transform_vector3(&self, other: $vec3) -> $vec3 {
             glam_assert!(self.row(3) == $vec4::W);
@@ -592,84 +661,7 @@ macro_rules! impl_mat4_traits {
             $mat4::from_cols(x_axis, y_axis, z_axis, w_axis)
         }
 
-        impl Default for $mat4 {
-            #[inline(always)]
-            fn default() -> Self {
-                Self::IDENTITY
-            }
-        }
-
-        impl PartialEq for $mat4 {
-            #[inline]
-            fn eq(&self, other: &Self) -> bool {
-                self.x_axis.eq(&other.x_axis)
-                    && self.y_axis.eq(&other.y_axis)
-                    && self.z_axis.eq(&other.z_axis)
-                    && self.w_axis.eq(&other.w_axis)
-            }
-        }
-
-        impl AsRef<[$t; 16]> for $mat4 {
-            #[inline]
-            fn as_ref(&self) -> &[$t; 16] {
-                unsafe { &*(self as *const Self as *const [$t; 16]) }
-            }
-        }
-
-        impl AsMut<[$t; 16]> for $mat4 {
-            #[inline]
-            fn as_mut(&mut self) -> &mut [$t; 16] {
-                unsafe { &mut *(self as *mut Self as *mut [$t; 16]) }
-            }
-        }
-
-        impl Add<$mat4> for $mat4 {
-            type Output = Self;
-            #[inline(always)]
-            fn add(self, other: Self) -> Self::Output {
-                self.add_mat4(&other)
-            }
-        }
-
-        impl Sub<$mat4> for $mat4 {
-            type Output = Self;
-            #[inline(always)]
-            fn sub(self, other: Self) -> Self::Output {
-                self.sub_mat4(&other)
-            }
-        }
-
-        impl Mul<$mat4> for $mat4 {
-            type Output = Self;
-            #[inline(always)]
-            fn mul(self, other: Self) -> Self::Output {
-                self.mul_mat4(&other)
-            }
-        }
-
-        impl Mul<$vec4> for $mat4 {
-            type Output = $vec4;
-            #[inline(always)]
-            fn mul(self, other: $vec4) -> Self::Output {
-                self.mul_vec4(other)
-            }
-        }
-
-        impl Mul<$mat4> for $t {
-            type Output = $mat4;
-            #[inline(always)]
-            fn mul(self, other: $mat4) -> Self::Output {
-                other.mul_scalar(self)
-            }
-        }
-
-        impl Mul<$t> for $mat4 {
-            type Output = Self;
-            #[inline(always)]
-            fn mul(self, other: $t) -> Self::Output {
-                self.mul_scalar(other)
-            }
-        }
+        impl_matn_common_traits!($t, $mat4, $vec4);
 
         impl Deref for $mat4 {
             type Target = Columns4<$vec4>;
@@ -686,9 +678,35 @@ macro_rules! impl_mat4_traits {
             }
         }
 
+        impl PartialEq for $mat4 {
+            #[inline]
+            fn eq(&self, other: &Self) -> bool {
+                self.x_axis.eq(&other.x_axis)
+                    && self.y_axis.eq(&other.y_axis)
+                    && self.z_axis.eq(&other.z_axis)
+                    && self.w_axis.eq(&other.w_axis)
+            }
+        }
+
+        #[cfg(not(target_arch = "spriv"))]
+        impl AsRef<[$t; 16]> for $mat4 {
+            #[inline]
+            fn as_ref(&self) -> &[$t; 16] {
+                unsafe { &*(self as *const Self as *const [$t; 16]) }
+            }
+        }
+
+        #[cfg(not(target_arch = "spriv"))]
+        impl AsMut<[$t; 16]> for $mat4 {
+            #[inline]
+            fn as_mut(&mut self) -> &mut [$t; 16] {
+                unsafe { &mut *(self as *mut Self as *mut [$t; 16]) }
+            }
+        }
+
         #[cfg(not(target_arch = "spirv"))]
         impl fmt::Debug for $mat4 {
-            fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+            fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
                 fmt.debug_struct(stringify!($mat4))
                     .field("x_axis", &self.x_axis)
                     .field("y_axis", &self.y_axis)
@@ -700,7 +718,7 @@ macro_rules! impl_mat4_traits {
 
         #[cfg(not(target_arch = "spirv"))]
         impl fmt::Display for $mat4 {
-            fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
                 write!(
                     f,
                     "[{}, {}, {}, {}]",
@@ -708,40 +726,26 @@ macro_rules! impl_mat4_traits {
                 )
             }
         }
-
-        #[cfg(feature = "std")]
-        impl<'a> Sum<&'a Self> for $mat4 {
-            fn sum<I>(iter: I) -> Self
-            where
-                I: Iterator<Item = &'a Self>,
-            {
-                iter.fold(Self::ZERO, |a, &b| Self::add(a, b))
-            }
-        }
-
-        #[cfg(feature = "std")]
-        impl<'a> Product<&'a Self> for $mat4 {
-            fn product<I>(iter: I) -> Self
-            where
-                I: Iterator<Item = &'a Self>,
-            {
-                iter.fold(Self::IDENTITY, |a, &b| Self::mul(a, b))
-            }
-        }
     };
 }
 
 #[cfg(all(target_feature = "sse2", not(feature = "scalar-math")))]
 type InnerF32 = Columns4<__m128>;
 
-#[cfg(any(not(target_feature = "sse2"), feature = "scalar-math"))]
+#[cfg(all(target_feature = "simd128", not(feature = "scalar-math")))]
+type InnerF32 = Columns4<v128>;
+
+#[cfg(any(
+    not(any(target_feature = "sse2", target_feature = "simd128")),
+    feature = "scalar-math"
+))]
 type InnerF32 = Columns4<XYZW<f32>>;
 
 /// A 4x4 column major matrix.
 ///
 /// This 4x4 matrix type features convenience methods for creating and using affine transforms and
 /// perspective projections. If you are primarily dealing with 3D affine transformations
-/// condidering using [`Affine3A`][crate::Affine3A] which is faster tha a 4x4 matrix for some
+/// considering using [`Affine3A`](crate::Affine3A) which is faster tha a 4x4 matrix for some
 /// affine operations.
 ///
 /// Affine transformations including 3D translation, rotation and scale can be created
@@ -776,7 +780,7 @@ pub struct Mat4(pub(crate) InnerF32);
 // define_mat4_struct!(Mat4, InnerF32);
 
 impl Mat4 {
-    impl_mat4_methods!(f32, Vec4, Vec3, Quat, InnerF32);
+    impl_mat4_methods!(f32, Vec4, Vec3, Mat3, Quat, InnerF32);
 
     /// Transforms the given `Vec3A` as 3D point.
     ///
@@ -796,13 +800,19 @@ impl Mat4 {
         Vec3A(self.0.transform_float4_as_vector3(other.0.into()).into())
     }
 
+    #[deprecated(since = "0.18.0", note = "please use `as_dmat4()` instead")]
     #[inline(always)]
     pub fn as_f64(&self) -> DMat4 {
+        self.as_dmat4()
+    }
+
+    #[inline(always)]
+    pub fn as_dmat4(&self) -> DMat4 {
         DMat4::from_cols(
-            self.x_axis.as_f64(),
-            self.y_axis.as_f64(),
-            self.z_axis.as_f64(),
-            self.w_axis.as_f64(),
+            self.x_axis.as_dvec4(),
+            self.y_axis.as_dvec4(),
+            self.z_axis.as_dvec4(),
+            self.w_axis.as_dvec4(),
         )
     }
 }
@@ -814,7 +824,7 @@ type InnerF64 = Columns4<XYZW<f64>>;
 ///
 /// This 4x4 matrix type features convenience methods for creating and using affine transforms and
 /// perspective projections. If you are primarily dealing with 3D affine transformations
-/// condidering using [`DAffine3`][crate::DAffine3] which is faster tha a 4x4 matrix for some
+/// considering using [`DAffine3`](crate::DAffine3) which is faster tha a 4x4 matrix for some
 /// affine operations.
 ///
 /// Affine transformations including 3D translation, rotation and scale can be created
@@ -845,15 +855,21 @@ pub struct DMat4(pub(crate) InnerF64);
 // define_mat4_struct!(DMat4, InnerF64);
 
 impl DMat4 {
-    impl_mat4_methods!(f64, DVec4, DVec3, DQuat, InnerF64);
+    impl_mat4_methods!(f64, DVec4, DVec3, DMat3, DQuat, InnerF64);
 
+    #[deprecated(since = "0.18.0", note = "please use `as_mat4()` instead")]
     #[inline(always)]
     pub fn as_f32(&self) -> Mat4 {
+        self.as_mat4()
+    }
+
+    #[inline(always)]
+    pub fn as_mat4(&self) -> Mat4 {
         Mat4::from_cols(
-            self.x_axis.as_f32(),
-            self.y_axis.as_f32(),
-            self.z_axis.as_f32(),
-            self.w_axis.as_f32(),
+            self.x_axis.as_vec4(),
+            self.y_axis.as_vec4(),
+            self.z_axis.as_vec4(),
+            self.w_axis.as_vec4(),
         )
     }
 }
@@ -861,7 +877,10 @@ impl_mat4_traits!(f64, dmat4, DMat4, DVec4);
 
 #[cfg(any(feature = "scalar-math", target_arch = "spriv"))]
 mod const_test_mat4 {
-    const_assert_eq!(4, core::mem::align_of::<super::Mat4>());
+    const_assert_eq!(
+        core::mem::align_of::<f32>(),
+        core::mem::align_of::<super::Mat4>()
+    );
     const_assert_eq!(64, core::mem::size_of::<super::Mat4>());
 }
 
@@ -872,6 +891,9 @@ mod const_test_mat4 {
 }
 
 mod const_test_dmat4 {
-    const_assert_eq!(8, core::mem::align_of::<super::DMat4>());
+    const_assert_eq!(
+        core::mem::align_of::<f64>(),
+        core::mem::align_of::<super::DMat4>()
+    );
     const_assert_eq!(128, core::mem::size_of::<super::DMat4>());
 }
