@@ -1,4 +1,5 @@
 // Adds common vector methods to an impl.
+
 // The methods here should be supported for all types of $t and all sizes of vector.
 macro_rules! impl_vecn_common_methods {
     ($t:ty, $vecn:ident, $mask:ident, $inner:ident, $vectrait:ident) => {
@@ -24,7 +25,7 @@ macro_rules! impl_vecn_common_methods {
             $vectrait::dot(self.0, other.0)
         }
 
-        /// Returns a vector containing the mininum values for each element of `self` and `other`.
+        /// Returns a vector containing the minimum values for each element of `self` and `other`.
         ///
         /// In other words this computes `[self.x.max(other.x), self.y.max(other.y), ..]`.
         #[inline(always)]
@@ -42,10 +43,11 @@ macro_rules! impl_vecn_common_methods {
 
         /// Component-wise clamping of values, similar to [`f32::clamp`].
         ///
-        /// Each element in `min` must be less-or-equal to the corresponing element in `max`.
+        /// Each element in `min` must be less-or-equal to the corresponding element in `max`.
         ///
-        /// If the `glam-assert` feature is enabled, the function will panic if the contract is not
-        /// met, otherwise the behavior is undefined.
+        /// # Panics
+        ///
+        /// Will panic if `min` is greater than `max` when `glam_assert` is enabled.
         #[inline(always)]
         pub fn clamp(self, min: Self, max: Self) -> Self {
             Self($vectrait::clamp(self.0, min.0, max.0))
@@ -133,7 +135,7 @@ macro_rules! impl_vecn_common_methods {
         ///
         /// Panics if `slice` is less than N elements long.
         #[inline(always)]
-        pub fn from_slice_unaligned(slice: &[$t]) -> Self {
+        pub fn from_slice(slice: &[$t]) -> Self {
             Self($vectrait::from_slice_unaligned(slice))
         }
 
@@ -143,7 +145,7 @@ macro_rules! impl_vecn_common_methods {
         ///
         /// Panics if `slice` is less than N elements long.
         #[inline(always)]
-        pub fn write_to_slice_unaligned(self, slice: &mut [$t]) {
+        pub fn write_to_slice(self, slice: &mut [$t]) {
             $vectrait::write_to_slice_unaligned(self.0, slice)
         }
 
@@ -186,6 +188,9 @@ macro_rules! impl_vecn_float_methods {
     ($t:ty, $vecn:ident, $mask:ident, $inner:ident, $flttrait:ident) => {
         // impl_vecn_signed_methods!($t, $vecn, $mask, $inner, $sgntrait, $vectrait);
 
+        /// All NAN.
+        pub const NAN: Self = Self(<$inner as crate::core::traits::scalar::NanConstEx>::NAN);
+
         /// Returns `true` if, and only if, all elements are finite.  If any element is either
         /// `NaN`, positive or negative infinity, this will return `false`.
         #[inline(always)]
@@ -208,7 +213,7 @@ macro_rules! impl_vecn_float_methods {
         }
 
         /// Computes the length of `self`.
-        #[cfg_attr(docsrs, doc(alias = "magnitude"))]
+        #[doc(alias = "magnitude")]
         #[inline(always)]
         pub fn length(self) -> $t {
             $flttrait::length(self.0)
@@ -217,7 +222,7 @@ macro_rules! impl_vecn_float_methods {
         /// Computes the squared length of `self`.
         ///
         /// This is faster than `length()` as it avoids a square root operation.
-        #[cfg_attr(docsrs, doc(alias = "magnitude2"))]
+        #[doc(alias = "magnitude2")]
         #[inline(always)]
         pub fn length_squared(self) -> $t {
             $flttrait::length_squared(self.0)
@@ -248,6 +253,11 @@ macro_rules! impl_vecn_float_methods {
         /// For valid results, `self` must _not_ be of length zero, nor very close to zero.
         ///
         /// See also [`Self::try_normalize`] and [`Self::normalize_or_zero`].
+        ///
+        /// Panics
+        ///
+        /// Will panic if `self` is zero length when `glam_assert` is enabled.
+        #[must_use]
         #[inline(always)]
         pub fn normalize(self) -> Self {
             Self($flttrait::normalize(self.0))
@@ -259,6 +269,7 @@ macro_rules! impl_vecn_float_methods {
         /// the result of this operation will be `None`.
         ///
         /// See also [`Self::normalize_or_zero`].
+        #[must_use]
         #[inline]
         pub fn try_normalize(self) -> Option<Self> {
             let rcp = self.length_recip();
@@ -275,6 +286,7 @@ macro_rules! impl_vecn_float_methods {
         /// the result of this operation will be zero.
         ///
         /// See also [`Self::try_normalize`].
+        #[must_use]
         #[inline]
         pub fn normalize_or_zero(self) -> Self {
             let rcp = self.length_recip();
@@ -293,6 +305,67 @@ macro_rules! impl_vecn_float_methods {
             $flttrait::is_normalized(self.0)
         }
 
+        /// Returns the vector projection of `self` onto `other`.
+        ///
+        /// `other` must be of non-zero length.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `other` is zero length when `glam_assert` is enabled.
+        #[must_use]
+        #[inline]
+        pub fn project_onto(self, other: Self) -> Self {
+            let other_len_sq_rcp = other.dot(other).recip();
+            glam_assert!(other_len_sq_rcp.is_finite());
+            other * self.dot(other) * other_len_sq_rcp
+        }
+
+        /// Returns the vector rejection of `self` from `other`.
+        ///
+        /// The vector rejection is the vector perpendicular to the projection of `self` onto
+        /// `other`, in other words the result of `self - self.project_onto(other)`.
+        ///
+        /// `other` must be of non-zero length.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `other` has a length of zero when `glam_assert` is enabled.
+        #[must_use]
+        #[inline]
+        pub fn reject_from(self, other: Self) -> Self {
+            self - self.project_onto(other)
+        }
+
+        /// Returns the vector projection of `self` onto `other`.
+        ///
+        /// `other` must be normalized.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `other` is not normalized when `glam_assert` is enabled.
+        #[must_use]
+        #[inline]
+        pub fn project_onto_normalized(self, other: Self) -> Self {
+            glam_assert!(other.is_normalized());
+            other * self.dot(other)
+        }
+
+        /// Returns the vector rejection of `self` from `other`.
+        ///
+        /// The vector rejection is the vector perpendicular to the projection of `self` onto
+        /// `other`, in other words the result of `self - self.project_onto(other)`.
+        ///
+        /// `other` must be normalized.
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `other` is not normalized when `glam_assert` is enabled.
+        #[must_use]
+        #[inline]
+        pub fn reject_from_normalized(self, other: Self) -> Self {
+            self - self.project_onto_normalized(other)
+        }
+
         /// Returns a vector containing the nearest integer to a number for each element of `self`.
         /// Round half-way cases away from 0.0.
         #[inline(always)]
@@ -314,6 +387,15 @@ macro_rules! impl_vecn_float_methods {
             Self($flttrait::ceil(self.0))
         }
 
+        /// Returns a vector containing the fractional part of the vector, e.g. `self -
+        /// self.floor()`.
+        ///
+        /// Note that this is fast but not precise for large numbers.
+        #[inline(always)]
+        pub fn fract(self) -> Self {
+            self - self.floor()
+        }
+
         /// Returns a vector containing `e^self` (the exponential function) for each element of
         /// `self`.
         #[inline(always)]
@@ -337,7 +419,7 @@ macro_rules! impl_vecn_float_methods {
         ///
         /// When `s` is `0.0`, the result will be equal to `self`.  When `s` is `1.0`, the result
         /// will be equal to `other`.
-        #[cfg_attr(docsrs, doc(alias = "mix"))]
+        #[doc(alias = "mix")]
         #[inline]
         pub fn lerp(self, other: Self, s: $t) -> Self {
             self + ((other - self) * s)
@@ -358,8 +440,13 @@ macro_rules! impl_vecn_float_methods {
         }
 
         /// Returns a vector with a length no less than `min` and no more than `max`
+        ///
+        /// # Panics
+        ///
+        /// Will panic if `min` is greater than `max` when `glam_assert` is enabled.
         #[inline]
         pub fn clamp_length(self, min: $t, max: $t) -> Self {
+            glam_assert!(min <= max);
             let length_sq = self.length_squared();
             if length_sq < min * min {
                 self * (length_sq.sqrt().recip() * min)
@@ -383,7 +470,7 @@ macro_rules! impl_vecn_float_methods {
         /// Returns a vector with a length no less than `min`
         pub fn clamp_length_min(self, min: $t) -> Self {
             let length_sq = self.length_squared();
-            if self.length_squared() < min * min {
+            if length_sq < min * min {
                 self * (length_sq.sqrt().recip() * min)
             } else {
                 self
@@ -500,22 +587,45 @@ macro_rules! impl_vecn_common_traits {
             }
         }
 
-        impl Add for $vecn {
+        impl Add<$vecn> for $vecn {
             type Output = Self;
             #[inline(always)]
-            fn add(self, other: Self) -> Self {
+            fn add(self, other: $vecn) -> Self {
                 Self(self.0.add(other.0))
             }
         }
 
-        impl AddAssign for $vecn {
+        impl AddAssign<$vecn> for $vecn {
             #[inline(always)]
-            fn add_assign(&mut self, other: Self) {
+            fn add_assign(&mut self, other: $vecn) {
                 self.0 = self.0.add(other.0)
             }
         }
 
-        impl Sub for $vecn {
+        impl Add<$t> for $vecn {
+            type Output = Self;
+            #[inline(always)]
+            fn add(self, other: $t) -> Self {
+                Self(self.0.add_scalar(other))
+            }
+        }
+
+        impl AddAssign<$t> for $vecn {
+            #[inline(always)]
+            fn add_assign(&mut self, other: $t) {
+                self.0 = self.0.add_scalar(other)
+            }
+        }
+
+        impl Add<$vecn> for $t {
+            type Output = $vecn;
+            #[inline(always)]
+            fn add(self, other: $vecn) -> $vecn {
+                $vecn($inner::splat(self).add(other.0))
+            }
+        }
+
+        impl Sub<$vecn> for $vecn {
             type Output = Self;
             #[inline(always)]
             fn sub(self, other: $vecn) -> Self {
@@ -523,13 +633,75 @@ macro_rules! impl_vecn_common_traits {
             }
         }
 
-        impl SubAssign for $vecn {
+        impl SubAssign<$vecn> for $vecn {
             #[inline(always)]
             fn sub_assign(&mut self, other: $vecn) {
                 self.0 = self.0.sub(other.0)
             }
         }
 
+        impl Sub<$t> for $vecn {
+            type Output = Self;
+            #[inline(always)]
+            fn sub(self, other: $t) -> Self {
+                Self(self.0.sub_scalar(other))
+            }
+        }
+
+        impl SubAssign<$t> for $vecn {
+            #[inline(always)]
+            fn sub_assign(&mut self, other: $t) {
+                self.0 = self.0.sub_scalar(other)
+            }
+        }
+
+        impl Sub<$vecn> for $t {
+            type Output = $vecn;
+            #[inline(always)]
+            fn sub(self, other: $vecn) -> $vecn {
+                $vecn($inner::splat(self).sub(other.0))
+            }
+        }
+
+        impl Rem<$vecn> for $vecn {
+            type Output = Self;
+            #[inline(always)]
+            fn rem(self, other: $vecn) -> Self {
+                Self(self.0.rem(other.0))
+            }
+        }
+
+        impl RemAssign<$vecn> for $vecn {
+            #[inline(always)]
+            fn rem_assign(&mut self, other: $vecn) {
+                self.0 = self.0.rem(other.0)
+            }
+        }
+
+        impl Rem<$t> for $vecn {
+            type Output = Self;
+            #[inline(always)]
+            fn rem(self, other: $t) -> Self {
+                Self(self.0.rem_scalar(other))
+            }
+        }
+
+        impl RemAssign<$t> for $vecn {
+            #[inline(always)]
+            fn rem_assign(&mut self, other: $t) {
+                self.0 = self.0.rem_scalar(other)
+            }
+        }
+
+        impl Rem<$vecn> for $t {
+            type Output = $vecn;
+            #[inline(always)]
+            fn rem(self, other: $vecn) -> $vecn {
+                $vecn($inner::splat(self).rem(other.0))
+            }
+        }
+
+        #[cfg(not(target_arch = "spriv"))]
         impl AsRef<[$t; $size]> for $vecn {
             #[inline(always)]
             fn as_ref(&self) -> &[$t; $size] {
@@ -537,6 +709,14 @@ macro_rules! impl_vecn_common_traits {
             }
         }
 
+        #[cfg(not(target_arch = "spriv"))]
+        impl AsMut<[$t; $size]> for $vecn {
+            #[inline(always)]
+            fn as_mut(&mut self) -> &mut [$t; $size] {
+                unsafe { &mut *(self as *mut $vecn as *mut [$t; $size]) }
+            }
+        }
+
         impl From<[$t; $size]> for $vecn {
             #[inline(always)]
             fn from(a: [$t; $size]) -> Self {
@@ -544,13 +724,6 @@ macro_rules! impl_vecn_common_traits {
             }
         }
 
-        impl AsMut<[$t; $size]> for $vecn {
-            #[inline(always)]
-            fn as_mut(&mut self) -> &mut [$t; $size] {
-                unsafe { &mut *(self as *mut $vecn as *mut [$t; $size]) }
-            }
-        }
-
         impl From<$vecn> for [$t; $size] {
             #[inline(always)]
             fn from(v: $vecn) -> Self {
@@ -558,22 +731,6 @@ macro_rules! impl_vecn_common_traits {
             }
         }
 
-        impl Index<usize> for $vecn {
-            type Output = $t;
-            #[inline(always)]
-            fn index(&self, index: usize) -> &Self::Output {
-                &self.as_ref()[index]
-            }
-        }
-
-        impl IndexMut<usize> for $vecn {
-            #[inline(always)]
-            fn index_mut(&mut self, index: usize) -> &mut Self::Output {
-                &mut self.as_mut()[index]
-            }
-        }
-
-        #[cfg(feature = "std")]
         impl<'a> Sum<&'a Self> for $vecn {
             #[inline]
             fn sum<I>(iter: I) -> Self
@@ -584,7 +741,6 @@ macro_rules! impl_vecn_common_traits {
             }
         }
 
-        #[cfg(feature = "std")]
         impl<'a> Product<&'a Self> for $vecn {
             #[inline]
             fn product<I>(iter: I) -> Self
@@ -601,6 +757,7 @@ macro_rules! impl_vecn_eq_hash_traits {
     ($t:ty, $size:literal, $vecn:ident) => {
         impl Eq for $vecn {}
 
+        #[cfg(not(target_arch = "spriv"))]
         impl core::hash::Hash for $vecn {
             fn hash<H: core::hash::Hasher>(&self, state: &mut H) {
                 let inner: &[$t; $size] = self.as_ref();
@@ -624,42 +781,315 @@ macro_rules! impl_vecn_signed_traits {
     };
 }
 
-macro_rules! impl_vecn_as_f32 {
-    ($ret:ident, $($e:ident),+) => {
+macro_rules! impl_vecn_shift_op_traits {
+    ($vecn:ident, $rhs:ty, $inner:ident) => {
+        impl Shl<$rhs> for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn shl(self, rhs: $rhs) -> Self::Output {
+                $vecn($inner::vector_shl(self.0, rhs.0))
+            }
+        }
+
+        impl Shr<$rhs> for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn shr(self, rhs: $rhs) -> Self::Output {
+                $vecn($inner::vector_shr(self.0, rhs.0))
+            }
+        }
+    };
+}
+
+macro_rules! impl_vecn_scalar_shift_op_traits {
+    ($vecn:ident, $rhs:ty, $inner:ident) => {
+        impl Shl<$rhs> for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn shl(self, rhs: $rhs) -> Self::Output {
+                $vecn($inner::scalar_shl(self.0, rhs))
+            }
+        }
+
+        impl Shr<$rhs> for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn shr(self, rhs: $rhs) -> Self::Output {
+                $vecn($inner::scalar_shr(self.0, rhs))
+            }
+        }
+    };
+}
+
+macro_rules! impl_vecn_bit_op_traits {
+    ($vecn:ident, $inner:ident) => {
+        impl Not for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn not(self) -> Self::Output {
+                $vecn($inner::not(self.0))
+            }
+        }
+
+        impl BitAnd for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn bitand(self, rhs: Self) -> Self::Output {
+                $vecn($inner::vector_bitand(self.0, rhs.0))
+            }
+        }
+
+        impl BitOr for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn bitor(self, rhs: Self) -> Self::Output {
+                $vecn($inner::vector_bitor(self.0, rhs.0))
+            }
+        }
+
+        impl BitXor for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn bitxor(self, rhs: Self) -> Self::Output {
+                $vecn($inner::vector_bitxor(self.0, rhs.0))
+            }
+        }
+    };
+}
+
+macro_rules! impl_vecn_scalar_bit_op_traits {
+    ($vecn:ident, $rhs:ty, $inner:ident) => {
+        impl BitAnd<$rhs> for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn bitand(self, rhs: $rhs) -> Self::Output {
+                $vecn($inner::scalar_bitand(self.0, rhs))
+            }
+        }
+
+        impl BitOr<$rhs> for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn bitor(self, rhs: $rhs) -> Self::Output {
+                $vecn($inner::scalar_bitor(self.0, rhs))
+            }
+        }
+
+        impl BitXor<$rhs> for $vecn {
+            type Output = Self;
+
+            #[inline(always)]
+            fn bitxor(self, rhs: $rhs) -> Self::Output {
+                $vecn($inner::scalar_bitxor(self.0, rhs))
+            }
+        }
+    };
+}
+
+macro_rules! impl_as_vec2 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_vec2()` instead")]
+        #[inline(always)]
+        pub fn as_f32(&self) -> Vec2 {
+            self.as_vec2()
+        }
+
         /// Casts all elements of `self` to `f32`.
         #[inline(always)]
-        pub fn as_f32(&self) -> $ret {
-            $ret::new($(self.$e as f32),+)
+        pub fn as_vec2(&self) -> Vec2 {
+            Vec2::new(self.x as f32, self.y as f32)
         }
     };
 }
 
-macro_rules! impl_vecn_as_f64 {
-    ($ret:ident, $($e:ident),+) => {
+macro_rules! impl_as_vec3 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_vec3()` instead")]
+        #[inline(always)]
+        pub fn as_f32(&self) -> Vec3 {
+            self.as_vec3()
+        }
+
+        /// Casts all elements of `self` to `f32`.
+        #[inline(always)]
+        pub fn as_vec3(&self) -> Vec3 {
+            Vec3::new(self.x as f32, self.y as f32, self.z as f32)
+        }
+
+        /// Casts all elements of `self` to `f32`.
+        #[inline(always)]
+        pub fn as_vec3a(&self) -> Vec3A {
+            Vec3A::new(self.x as f32, self.y as f32, self.z as f32)
+        }
+    };
+}
+
+macro_rules! impl_as_vec4 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_vec4()` instead")]
+        #[inline(always)]
+        pub fn as_f32(&self) -> Vec4 {
+            self.as_vec4()
+        }
+
+        /// Casts all elements of `self` to `f32`.
+        #[inline(always)]
+        pub fn as_vec4(&self) -> Vec4 {
+            Vec4::new(self.x as f32, self.y as f32, self.z as f32, self.w as f32)
+        }
+    };
+}
+
+macro_rules! impl_as_dvec2 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_dvec2()` instead")]
+        #[inline(always)]
+        pub fn as_f64(&self) -> DVec2 {
+            self.as_dvec2()
+        }
+
         /// Casts all elements of `self` to `f64`.
         #[inline(always)]
-        pub fn as_f64(&self) -> $ret {
-            $ret::new($(self.$e as f64),+)
+        pub fn as_dvec2(&self) -> DVec2 {
+            DVec2::new(self.x as f64, self.y as f64)
         }
     };
 }
 
-macro_rules! impl_vecn_as_i32 {
-    ($ret:ident, $($e:ident),+) => {
+macro_rules! impl_as_dvec3 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_dvec3()` instead")]
+        #[inline(always)]
+        pub fn as_f64(&self) -> DVec3 {
+            self.as_dvec3()
+        }
+
+        /// Casts all elements of `self` to `f64`.
+        #[inline(always)]
+        pub fn as_dvec3(&self) -> DVec3 {
+            DVec3::new(self.x as f64, self.y as f64, self.z as f64)
+        }
+    };
+}
+
+macro_rules! impl_as_dvec4 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_dvec4()` instead")]
+        #[inline(always)]
+        pub fn as_f64(&self) -> DVec4 {
+            self.as_dvec4()
+        }
+
+        /// Casts all elements of `self` to `f64`.
+        #[inline(always)]
+        pub fn as_dvec4(&self) -> DVec4 {
+            DVec4::new(self.x as f64, self.y as f64, self.z as f64, self.w as f64)
+        }
+    };
+}
+
+macro_rules! impl_as_ivec2 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_ivec2()` instead")]
+        #[inline(always)]
+        pub fn as_i32(&self) -> IVec2 {
+            self.as_ivec2()
+        }
+
         /// Casts all elements of `self` to `i32`.
         #[inline(always)]
-        pub fn as_i32(&self) -> $ret {
-            $ret::new($(self.$e as i32),+)
+        pub fn as_ivec2(&self) -> IVec2 {
+            IVec2::new(self.x as i32, self.y as i32)
         }
     };
 }
 
-macro_rules! impl_vecn_as_u32 {
-    ($ret:ident, $($e:ident),+) => {
-        /// Casts all elements of `self` to `u32`.
+macro_rules! impl_as_ivec3 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_ivec3()` instead")]
         #[inline(always)]
-        pub fn as_u32(&self) -> $ret {
-            $ret::new($(self.$e as u32),+)
+        pub fn as_i32(&self) -> IVec3 {
+            self.as_ivec3()
+        }
+
+        /// Casts all elements of `self` to `i32`.
+        #[inline(always)]
+        pub fn as_ivec3(&self) -> IVec3 {
+            IVec3::new(self.x as i32, self.y as i32, self.z as i32)
+        }
+    };
+}
+
+macro_rules! impl_as_ivec4 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_ivec4()` instead")]
+        #[inline(always)]
+        pub fn as_i32(&self) -> IVec4 {
+            self.as_ivec4()
+        }
+
+        /// Casts all elements of `self` to `i32`.
+        #[inline(always)]
+        pub fn as_ivec4(&self) -> IVec4 {
+            IVec4::new(self.x as i32, self.y as i32, self.z as i32, self.w as i32)
+        }
+    };
+}
+
+macro_rules! impl_as_uvec2 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_uvec2()` instead")]
+        #[inline(always)]
+        pub fn as_u32(&self) -> UVec2 {
+            self.as_uvec2()
+        }
+
+        /// Casts all elements of `self` to `u32`.
+        #[inline(always)]
+        pub fn as_uvec2(&self) -> UVec2 {
+            UVec2::new(self.x as u32, self.y as u32)
+        }
+    };
+}
+
+macro_rules! impl_as_uvec3 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_uvec3()` instead")]
+        #[inline(always)]
+        pub fn as_u32(&self) -> UVec3 {
+            self.as_uvec3()
+        }
+
+        /// Casts all elements of `self` to `u32`.
+        #[inline(always)]
+        pub fn as_uvec3(&self) -> UVec3 {
+            UVec3::new(self.x as u32, self.y as u32, self.z as u32)
+        }
+    };
+}
+
+macro_rules! impl_as_uvec4 {
+    () => {
+        #[deprecated(since = "0.18.0", note = "please use `as_uvec4()` instead")]
+        #[inline(always)]
+        pub fn as_u32(&self) -> UVec4 {
+            self.as_uvec4()
+        }
+
+        /// Casts all elements of `self` to `u32`.
+        #[inline(always)]
+        pub fn as_uvec4(&self) -> UVec4 {
+            UVec4::new(self.x as u32, self.y as u32, self.z as u32, self.w as u32)
         }
     };
 }
