@@ -3,14 +3,12 @@ mod support;
 
 macro_rules! impl_vec3_tests {
     ($t:ident, $const_new:ident, $new:ident, $vec3:ident, $mask:ident) => {
-        #[test]
-        fn test_const() {
+        glam_test!(test_const, {
             const V: $vec3 = $const_new!([1 as $t, 2 as $t, 3 as $t]);
             assert_eq!($vec3::new(1 as $t, 2 as $t, 3 as $t), V);
-        }
+        });
 
-        #[test]
-        fn test_new() {
+        glam_test!(test_new, {
             let v = $new(1 as $t, 2 as $t, 3 as $t);
 
             assert_eq!(v.x, 1 as $t);
@@ -32,10 +30,9 @@ macro_rules! impl_vec3_tests {
             assert_eq!($vec3::new(1 as $t, 0 as $t, 0 as $t), $vec3::X);
             assert_eq!($vec3::new(0 as $t, 1 as $t, 0 as $t), $vec3::Y);
             assert_eq!($vec3::new(0 as $t, 0 as $t, 1 as $t), $vec3::Z);
-        }
+        });
 
-        #[test]
-        fn test_fmt() {
+        glam_test!(test_fmt, {
             let a = $vec3::new(1 as $t, 2 as $t, 3 as $t);
             assert_eq!(
                 format!("{:?}", a),
@@ -43,23 +40,20 @@ macro_rules! impl_vec3_tests {
             );
             // assert_eq!(format!("{:#?}", a), "$vec3(\n    1.0,\n    2.0,\n    3.0\n)");
             assert_eq!(format!("{}", a), "[1, 2, 3]");
-        }
+        });
 
-        #[test]
-        fn test_zero() {
+        glam_test!(test_zero, {
             let v = $vec3::ZERO;
             assert_eq!((0 as $t, 0 as $t, 0 as $t), v.into());
             assert_eq!(v, $vec3::default());
-        }
+        });
 
-        #[test]
-        fn test_splat() {
+        glam_test!(test_splat, {
             let v = $vec3::splat(1 as $t);
             assert_eq!($vec3::ONE, v);
-        }
+        });
 
-        #[test]
-        fn test_accessors() {
+        glam_test!(test_accessors, {
             let mut a = $vec3::ZERO;
             a.x = 1 as $t;
             a.y = 2 as $t;
@@ -77,68 +71,72 @@ macro_rules! impl_vec3_tests {
             assert_eq!(2 as $t, a[1]);
             assert_eq!(3 as $t, a[2]);
             assert_eq!((1 as $t, 2 as $t, 3 as $t), a.into());
-        }
+        });
 
-        #[test]
-        fn test_dot_unsigned() {
+        glam_test!(test_dot_unsigned, {
             let x = $new(1 as $t, 0 as $t, 0 as $t);
             let y = $new(0 as $t, 1 as $t, 0 as $t);
             assert_eq!(1 as $t, x.dot(x));
             assert_eq!(0 as $t, x.dot(y));
-        }
+        });
 
-        #[test]
-        fn test_cross() {
+        glam_test!(test_cross, {
             let x = $new(1 as $t, 0 as $t, 0 as $t);
             let y = $new(0 as $t, 1 as $t, 0 as $t);
             let z = $new(0 as $t, 0 as $t, 1 as $t);
             assert_eq!(y, z.cross(x));
             assert_eq!(z, x.cross(y));
-        }
+        });
 
-        #[test]
-        fn test_ops() {
+        glam_test!(test_ops, {
             let a = $new(2 as $t, 4 as $t, 8 as $t);
-            assert_eq!((4 as $t, 8 as $t, 16 as $t), (a + a).into());
-            assert_eq!((0 as $t, 0 as $t, 0 as $t), (a - a).into());
-            assert_eq!((4 as $t, 16 as $t, 64 as $t), (a * a).into());
-            assert_eq!((4 as $t, 8 as $t, 16 as $t), (a * 2 as $t).into());
-            assert_eq!((4 as $t, 8 as $t, 16 as $t), (2 as $t * a).into());
-            assert_eq!((1 as $t, 1 as $t, 1 as $t), (a / a).into());
-            assert_eq!((1 as $t, 2 as $t, 4 as $t), (a / 2 as $t).into());
-            assert_eq!((4 as $t, 2 as $t, 1 as $t), (8 as $t / a).into());
-        }
+            assert_eq!($new(4 as $t, 8 as $t, 16 as $t), a + a);
+            assert_eq!($new(0 as $t, 0 as $t, 0 as $t), a - a);
+            assert_eq!($new(4 as $t, 16 as $t, 64 as $t), a * a);
+            assert_eq!($new(4 as $t, 8 as $t, 16 as $t), a * 2 as $t);
+            assert_eq!($new(4 as $t, 8 as $t, 16 as $t), 2 as $t * a);
+            assert_eq!($new(1 as $t, 1 as $t, 1 as $t), a / a);
+            assert_eq!($new(1 as $t, 2 as $t, 4 as $t), a / 2 as $t);
+            assert_eq!($new(4 as $t, 2 as $t, 1 as $t), 8 as $t / a);
+            assert_eq!($new(0 as $t, 0 as $t, 0 as $t), a % a);
+            assert_eq!($new(0 as $t, 1 as $t, 1 as $t), a % (a - 1 as $t));
+            assert_eq!($new(0 as $t, 0 as $t, 0 as $t), a % 1 as $t);
+            assert_eq!($new(2 as $t, 1 as $t, 2 as $t), a % 3 as $t);
+            assert_eq!($new(1 as $t, 1 as $t, 1 as $t), 17 as $t % a);
+            assert_eq!($new(2 as $t, 4 as $t, 0 as $t), a % 8 as $t);
+        });
 
-        #[test]
-        fn test_assign_ops() {
+        glam_test!(test_assign_ops, {
             let a = $new(1 as $t, 2 as $t, 3 as $t);
             let mut b = a;
             b += a;
-            assert_eq!((2 as $t, 4 as $t, 6 as $t), b.into());
+            assert_eq!($new(2 as $t, 4 as $t, 6 as $t), b);
             b -= a;
-            assert_eq!((1 as $t, 2 as $t, 3 as $t), b.into());
+            assert_eq!($new(1 as $t, 2 as $t, 3 as $t), b);
             b *= a;
-            assert_eq!((1 as $t, 4 as $t, 9 as $t), b.into());
+            assert_eq!($new(1 as $t, 4 as $t, 9 as $t), b);
             b /= a;
-            assert_eq!((1 as $t, 2 as $t, 3 as $t), b.into());
+            assert_eq!($new(1 as $t, 2 as $t, 3 as $t), b);
             b *= 2 as $t;
-            assert_eq!((2 as $t, 4 as $t, 6 as $t), b.into());
+            assert_eq!($new(2 as $t, 4 as $t, 6 as $t), b);
             b /= 2 as $t;
-            assert_eq!((1 as $t, 2 as $t, 3 as $t), b.into());
-        }
+            assert_eq!($new(1 as $t, 2 as $t, 3 as $t), b);
+            b %= (b + 1 as $t);
+            assert_eq!($new(1 as $t, 2 as $t, 3 as $t), b);
+            b %= b;
+            assert_eq!($new(0 as $t, 0 as $t, 0 as $t), b);
+        });
 
-        #[test]
-        fn test_min_max() {
+        glam_test!(test_min_max, {
             let a = $new(3 as $t, 5 as $t, 1 as $t);
             let b = $new(4 as $t, 2 as $t, 6 as $t);
             assert_eq!((3 as $t, 2 as $t, 1 as $t), a.min(b).into());
             assert_eq!((3 as $t, 2 as $t, 1 as $t), b.min(a).into());
             assert_eq!((4 as $t, 5 as $t, 6 as $t), a.max(b).into());
             assert_eq!((4 as $t, 5 as $t, 6 as $t), b.max(a).into());
-        }
+        });
 
-        #[test]
-        fn test_clamp() {
+        glam_test!(test_clamp, {
             fn vec(x: i32, y: i32, z: i32) -> $vec3 {
                 $vec3::new(x as $t, y as $t, z as $t)
             }
@@ -152,17 +150,15 @@ macro_rules! impl_vec3_tests {
             assert_eq!(vec(9, 9, 9).clamp(min, max), vec(6, 8, 8));
 
             should_glam_assert!({ $vec3::clamp($vec3::ZERO, $vec3::ONE, $vec3::ZERO) });
-        }
+        });
 
-        #[test]
-        fn test_hmin_hmax() {
+        glam_test!(test_hmin_hmax, {
             let a = $new(2 as $t, 3 as $t, 1 as $t);
             assert_eq!(1 as $t, a.min_element());
             assert_eq!(3 as $t, a.max_element());
-        }
+        });
 
-        #[test]
-        fn test_eq() {
+        glam_test!(test_eq, {
             let a = $new(1 as $t, 1 as $t, 1 as $t);
             let b = $new(1 as $t, 2 as $t, 3 as $t);
             assert!(a.cmpeq(a).all());
@@ -170,10 +166,9 @@ macro_rules! impl_vec3_tests {
             assert!(a.cmpne(b).any());
             assert!(b.cmpne(a).any());
             assert!(b.cmpeq(a).any());
-        }
+        });
 
-        #[test]
-        fn test_cmp() {
+        glam_test!(test_cmp, {
             assert!(!$mask::default().any());
             assert!(!$mask::default().all());
             assert_eq!($mask::default().bitmask(), 0x0);
@@ -197,26 +192,24 @@ macro_rules! impl_vec3_tests {
             assert!(c.cmple(c).all());
             assert!(c.cmpge(c).all());
             assert!(a == a);
-        }
+        });
 
-        #[test]
-        fn test_extend_truncate() {
+        glam_test!(test_extend_truncate, {
             let a = $new(1 as $t, 2 as $t, 3 as $t);
             let b = a.extend(4 as $t);
             assert_eq!((1 as $t, 2 as $t, 3 as $t, 4 as $t), b.into());
             let c = $vec3::from(b.truncate());
             assert_eq!(a, c);
-        }
+        });
 
-        #[test]
-        fn test_mask() {
+        glam_test!(test_mask, {
             let mut a = $vec3::ZERO;
             a.x = 1 as $t;
             a.y = 1 as $t;
             a.z = 1 as $t;
             assert!(!a.cmpeq($vec3::ZERO).any());
             assert!(a.cmpeq($vec3::ONE).all());
-        }
+        });
 
         // #[test]
         // fn test_mask_as_ref() {
@@ -228,8 +221,7 @@ macro_rules! impl_vec3_tests {
         //     assert_eq!($mask::new(true, true, true).as_ref(), &[!0, !0, !0]);
         // }
 
-        #[test]
-        fn test_mask_from() {
+        glam_test!(test_mask_from, {
             assert_eq!(
                 Into::<[u32; 3]>::into($mask::new(false, false, false)),
                 [0, 0, 0]
@@ -254,36 +246,32 @@ macro_rules! impl_vec3_tests {
                 Into::<[u32; 3]>::into($mask::new(true, true, true)),
                 [!0, !0, !0]
             );
-        }
+        });
 
-        #[test]
-        fn test_mask_bitmask() {
+        glam_test!(test_mask_bitmask, {
             assert_eq!($mask::new(false, false, false).bitmask(), 0b000);
             assert_eq!($mask::new(true, false, false).bitmask(), 0b001);
             assert_eq!($mask::new(false, true, true).bitmask(), 0b110);
             assert_eq!($mask::new(false, true, false).bitmask(), 0b010);
             assert_eq!($mask::new(true, false, true).bitmask(), 0b101);
             assert_eq!($mask::new(true, true, true).bitmask(), 0b111);
-        }
+        });
 
-        #[test]
-        fn test_mask_any() {
+        glam_test!(test_mask_any, {
             assert_eq!($mask::new(false, false, false).any(), false);
             assert_eq!($mask::new(true, false, false).any(), true);
             assert_eq!($mask::new(false, true, false).any(), true);
             assert_eq!($mask::new(false, false, true).any(), true);
-        }
+        });
 
-        #[test]
-        fn test_mask_all() {
+        glam_test!(test_mask_all, {
             assert_eq!($mask::new(true, true, true).all(), true);
             assert_eq!($mask::new(false, true, true).all(), false);
             assert_eq!($mask::new(true, false, true).all(), false);
             assert_eq!($mask::new(true, true, false).all(), false);
-        }
+        });
 
-        #[test]
-        fn test_mask_select() {
+        glam_test!(test_mask_select, {
             let a = $vec3::new(1 as $t, 2 as $t, 3 as $t);
             let b = $vec3::new(4 as $t, 5 as $t, 6 as $t);
             assert_eq!(
@@ -302,10 +290,9 @@ macro_rules! impl_vec3_tests {
                 $vec3::select($mask::new(false, false, false), a, b),
                 $vec3::new(4 as $t, 5 as $t, 6 as $t),
             );
-        }
+        });
 
-        #[test]
-        fn test_mask_and() {
+        glam_test!(test_mask_and, {
             assert_eq!(
                 ($mask::new(false, false, false) & $mask::new(false, false, false)).bitmask(),
                 0b000,
@@ -326,10 +313,9 @@ macro_rules! impl_vec3_tests {
             let mut mask = $mask::new(true, true, false);
             mask &= $mask::new(true, false, false);
             assert_eq!(mask.bitmask(), 0b001);
-        }
+        });
 
-        #[test]
-        fn test_mask_or() {
+        glam_test!(test_mask_or, {
             assert_eq!(
                 ($mask::new(false, false, false) | $mask::new(false, false, false)).bitmask(),
                 0b000,
@@ -350,18 +336,16 @@ macro_rules! impl_vec3_tests {
             let mut mask = $mask::new(true, true, false);
             mask |= $mask::new(true, false, false);
             assert_eq!(mask.bitmask(), 0b011);
-        }
+        });
 
-        #[test]
-        fn test_mask_not() {
+        glam_test!(test_mask_not, {
             assert_eq!((!$mask::new(false, false, false)).bitmask(), 0b111);
             assert_eq!((!$mask::new(true, true, true)).bitmask(), 0b000);
             assert_eq!((!$mask::new(true, false, true)).bitmask(), 0b010);
             assert_eq!((!$mask::new(false, true, false)).bitmask(), 0b101);
-        }
+        });
 
-        #[test]
-        fn test_mask_fmt() {
+        glam_test!(test_mask_fmt, {
             let a = $mask::new(true, false, false);
 
             // // debug fmt
@@ -372,10 +356,9 @@ macro_rules! impl_vec3_tests {
 
             // display fmt
             assert_eq!(format!("{}", a), "[true, false, false]");
-        }
+        });
 
-        #[test]
-        fn test_mask_eq() {
+        glam_test!(test_mask_eq, {
             let a = $mask::new(true, false, true);
             let b = $mask::new(true, false, true);
             let c = $mask::new(false, true, true);
@@ -384,10 +367,9 @@ macro_rules! impl_vec3_tests {
             assert_eq!(b, a);
             assert_ne!(a, c);
             assert_ne!(b, c);
-        }
+        });
 
-        #[test]
-        fn test_mask_hash() {
+        glam_test!(test_mask_hash, {
             use std::collections::hash_map::DefaultHasher;
             use std::hash::Hash;
             use std::hash::Hasher;
@@ -412,10 +394,9 @@ macro_rules! impl_vec3_tests {
             assert_eq!(a_hashed, b_hashed);
             assert_ne!(a, c);
             assert_ne!(a_hashed, c_hashed);
-        }
+        });
 
-        #[test]
-        fn test_to_from_slice() {
+        glam_test!(test_to_from_slice, {
             let v = $vec3::new(1 as $t, 2 as $t, 3 as $t);
             let mut a = [0 as $t, 0 as $t, 0 as $t];
             v.write_to_slice(&mut a);
@@ -423,21 +404,17 @@ macro_rules! impl_vec3_tests {
 
             should_panic!({ $vec3::ONE.write_to_slice(&mut [0 as $t; 2]) });
             should_panic!({ $vec3::from_slice(&[0 as $t; 2]) });
-        }
+        });
 
-        #[cfg(feature = "std")]
-        #[test]
-        fn test_sum() {
+        glam_test!(test_sum, {
             let one = $vec3::ONE;
             assert_eq!(vec![one, one].iter().sum::<$vec3>(), one + one);
-        }
+        });
 
-        #[cfg(feature = "std")]
-        #[test]
-        fn test_product() {
+        glam_test!(test_product, {
             let two = $vec3::new(2 as $t, 2 as $t, 2 as $t);
             assert_eq!(vec![two, two].iter().product::<$vec3>(), two * two);
-        }
+        });
     };
 }
 
@@ -445,28 +422,25 @@ macro_rules! impl_vec3_signed_tests {
     ($t:ident, $const_new:ident, $new:ident, $vec3:ident, $mask:ident) => {
         impl_vec3_tests!($t, $const_new, $new, $vec3, $mask);
 
-        #[test]
-        fn test_neg() {
+        glam_test!(test_neg, {
             let a = $new(1 as $t, 2 as $t, 3 as $t);
             assert_eq!((-1 as $t, -2 as $t, -3 as $t), (-a).into());
-        }
+        });
 
-        #[test]
-        fn test_dot_signed() {
+        glam_test!(test_dot_signed, {
             let x = $new(1 as $t, 0 as $t, 0 as $t);
             let y = $new(0 as $t, 1 as $t, 0 as $t);
             let z = $new(0 as $t, 0 as $t, 1 as $t);
             assert_eq!(1 as $t, x.dot(x));
             assert_eq!(0 as $t, x.dot(y));
             assert_eq!(-1 as $t, z.dot(-z));
-        }
+        });
     };
 }
 
 macro_rules! impl_vec3_eq_hash_tests {
     ($t:ident, $new:ident) => {
-        #[test]
-        fn test_vec3_hash() {
+        glam_test!(test_vec3_hash, {
             use std::collections::hash_map::DefaultHasher;
             use std::hash::Hash;
             use std::hash::Hasher;
@@ -491,7 +465,7 @@ macro_rules! impl_vec3_eq_hash_tests {
             assert_eq!(a_hashed, b_hashed);
             assert_ne!(a, c);
             assert_ne!(a_hashed, c_hashed);
-        }
+        });
     };
 }
 
@@ -504,17 +478,20 @@ macro_rules! impl_vec3_float_tests {
         use core::$t::NAN;
         use core::$t::NEG_INFINITY;
 
-        #[test]
-        fn test_vec3_consts() {
+        glam_test!(test_nan, {
+            assert!($vec3::NAN.is_nan());
+            assert!(!$vec3::NAN.is_finite());
+        });
+
+        glam_test!(test_vec3_consts, {
             assert_eq!($vec3::ZERO, $new(0 as $t, 0 as $t, 0 as $t));
             assert_eq!($vec3::ONE, $new(1 as $t, 1 as $t, 1 as $t));
             assert_eq!($vec3::X, $new(1 as $t, 0 as $t, 0 as $t));
             assert_eq!($vec3::Y, $new(0 as $t, 1 as $t, 0 as $t));
             assert_eq!($vec3::Z, $new(0 as $t, 0 as $t, 1 as $t));
-        }
+        });
 
-        #[test]
-        fn test_funcs() {
+        glam_test!(test_funcs, {
             let x = $new(1.0, 0.0, 0.0);
             let y = $new(0.0, 1.0, 0.0);
             let z = $new(0.0, 0.0, 1.0);
@@ -554,10 +531,9 @@ macro_rules! impl_vec3_float_tests {
                 $new(2.0, 3.0, 4.0).normalize()
             );
             assert_eq!($new(0.5, 0.25, 0.125), $new(2.0, 4.0, 8.0).recip());
-        }
+        });
 
-        #[test]
-        fn test_project_reject() {
+        glam_test!(test_project_reject, {
             assert_eq!(
                 $new(0.0, 0.0, 1.0),
                 $new(1.0, 0.0, 1.0).project_onto($new(0.0, 0.0, 2.0))
@@ -578,10 +554,9 @@ macro_rules! impl_vec3_float_tests {
             should_glam_assert!({ $vec3::ONE.reject_from($vec3::ZERO) });
             should_glam_assert!({ $vec3::ONE.project_onto_normalized($vec3::ONE) });
             should_glam_assert!({ $vec3::ONE.reject_from_normalized($vec3::ONE) });
-        }
+        });
 
-        #[test]
-        fn test_signum() {
+        glam_test!(test_signum, {
             assert_eq!($vec3::ZERO.signum(), $vec3::ONE);
             assert_eq!(-$vec3::ZERO.signum(), -$vec3::ONE);
             assert_eq!($vec3::ONE.signum(), $vec3::ONE);
@@ -589,17 +564,15 @@ macro_rules! impl_vec3_float_tests {
             assert_eq!($vec3::splat(INFINITY).signum(), $vec3::ONE);
             assert_eq!($vec3::splat(NEG_INFINITY).signum(), -$vec3::ONE);
             assert!($vec3::splat(NAN).signum().is_nan_mask().all());
-        }
+        });
 
-        #[test]
-        fn test_abs() {
+        glam_test!(test_abs, {
             assert_eq!($vec3::ZERO.abs(), $vec3::ZERO);
             assert_eq!($vec3::ONE.abs(), $vec3::ONE);
             assert_eq!((-$vec3::ONE).abs(), $vec3::ONE);
-        }
+        });
 
-        #[test]
-        fn test_round() {
+        glam_test!(test_round, {
             assert_eq!($vec3::new(1.35, 0.0, 0.0).round().x, 1.0);
             assert_eq!($vec3::new(0.0, 1.5, 0.0).round().y, 2.0);
             assert_eq!($vec3::new(0.0, 0.0, -15.5).round().z, -16.0);
@@ -612,10 +585,9 @@ macro_rules! impl_vec3_float_tests {
                 $vec3::new(NEG_INFINITY, INFINITY, 0.0)
             );
             assert!($vec3::new(NAN, 0.0, 0.0).round().x.is_nan());
-        }
+        });
 
-        #[test]
-        fn test_floor() {
+        glam_test!(test_floor, {
             assert_eq!(
                 $vec3::new(1.35, 1.5, -1.5).floor(),
                 $vec3::new(1.0, 1.0, -2.0)
@@ -629,10 +601,9 @@ macro_rules! impl_vec3_float_tests {
                 $vec3::new(-2000000.123, 10000000.123, 1000.9).floor(),
                 $vec3::new(-2000001.0, 10000000.0, 1000.0)
             );
-        }
+        });
 
-        #[test]
-        fn test_fract() {
+        glam_test!(test_fract, {
             assert_approx_eq!(
                 $vec3::new(1.35, 1.5, -1.5).fract(),
                 $vec3::new(0.35, 0.5, 0.5)
@@ -642,9 +613,9 @@ macro_rules! impl_vec3_float_tests {
                 $vec3::new(0.877, 0.123, 0.9),
                 0.002
             );
-        }
-        #[test]
-        fn test_ceil() {
+        });
+
+        glam_test!(test_ceil, {
             assert_eq!(
                 $vec3::new(1.35, 1.5, -1.5).ceil(),
                 $vec3::new(2.0, 2.0, -1.0)
@@ -658,44 +629,40 @@ macro_rules! impl_vec3_float_tests {
                 $vec3::new(-2000000.123, 1000000.123, 1000.9).ceil(),
                 $vec3::new(-2000000.0, 1000001.0, 1001.0)
             );
-        }
+        });
 
-        #[test]
-        fn test_lerp() {
+        glam_test!(test_lerp, {
             let v0 = $vec3::new(-1.0, -1.0, -1.0);
             let v1 = $vec3::new(1.0, 1.0, 1.0);
             assert_approx_eq!(v0, v0.lerp(v1, 0.0));
             assert_approx_eq!(v1, v0.lerp(v1, 1.0));
             assert_approx_eq!($vec3::ZERO, v0.lerp(v1, 0.5));
-        }
-        #[test]
-        fn test_is_finite() {
+        });
+
+        glam_test!(test_is_finite, {
             assert!($vec3::new(0.0, 0.0, 0.0).is_finite());
             assert!($vec3::new(-1e-10, 1.0, 1e10).is_finite());
             assert!(!$vec3::new(INFINITY, 0.0, 0.0).is_finite());
             assert!(!$vec3::new(0.0, NAN, 0.0).is_finite());
             assert!(!$vec3::new(0.0, 0.0, NEG_INFINITY).is_finite());
             assert!(!$vec3::splat(NAN).is_finite());
-        }
+        });
 
-        #[test]
-        fn test_powf() {
+        glam_test!(test_powf, {
             assert_eq!(
                 $vec3::new(2.0, 4.0, 8.0).powf(2.0),
                 $vec3::new(4.0, 16.0, 64.0)
             );
-        }
+        });
 
-        #[test]
-        fn test_exp() {
+        glam_test!(test_exp, {
             assert_eq!(
                 $vec3::new(1.0, 2.0, 3.0).exp(),
                 $vec3::new((1.0 as $t).exp(), (2.0 as $t).exp(), (3.0 as $t).exp())
             );
-        }
+        });
 
-        #[test]
-        fn test_angle_between() {
+        glam_test!(test_angle_between, {
             let angle = $vec3::new(1.0, 0.0, 1.0).angle_between($vec3::new(1.0, 1.0, 0.0));
             assert_approx_eq!(core::$t::consts::FRAC_PI_3, angle, 1e-6);
 
@@ -704,10 +671,9 @@ macro_rules! impl_vec3_float_tests {
 
             let angle = $vec3::new(-1.0, 0.0, -1.0).angle_between($vec3::new(1.0, -1.0, 0.0));
             assert_approx_eq!(2.0 * core::$t::consts::FRAC_PI_3, angle, 1e-6);
-        }
+        });
 
-        #[test]
-        fn test_clamp_length() {
+        glam_test!(test_clamp_length, {
             // Too long gets shortened
             assert_eq!(
                 $vec3::new(12.0, 16.0, 0.0).clamp_length(7.0, 10.0),
@@ -724,10 +690,9 @@ macro_rules! impl_vec3_float_tests {
                 $vec3::new(6.0, 8.0, 0.0) // lengthened to length 10.0
             );
             should_glam_assert!({ $vec3::ONE.clamp_length(1.0, 0.0) });
-        }
+        });
 
-        #[test]
-        fn test_clamp_length_max() {
+        glam_test!(test_clamp_length_max, {
             // Too long gets shortened
             assert_eq!(
                 $vec3::new(12.0, 16.0, 0.0).clamp_length_max(10.0),
@@ -738,10 +703,9 @@ macro_rules! impl_vec3_float_tests {
                 $vec3::new(2.0, 1.0, 0.0).clamp_length_max(5.0),
                 $vec3::new(2.0, 1.0, 0.0) // unchanged
             );
-        }
+        });
 
-        #[test]
-        fn test_clamp_length_min() {
+        glam_test!(test_clamp_length_min, {
             // Not too short is unchanged
             assert_eq!(
                 $vec3::new(2.0, 1.0, 0.0).clamp_length_min(0.5),
@@ -752,11 +716,10 @@ macro_rules! impl_vec3_float_tests {
                 $vec3::new(0.6, 0.8, 0.0).clamp_length_min(10.0),
                 $vec3::new(6.0, 8.0, 0.0) // lengthened to length 10.0
             );
-        }
+        });
 
         #[cfg(feature = "std")]
-        #[test]
-        fn test_any_ortho() {
+        glam_test!(test_any_ortho, {
             let eps = 2.0 * core::$t::EPSILON;
 
             for &v in &vec3_float_test_vectors!($vec3) {
@@ -774,63 +737,228 @@ macro_rules! impl_vec3_float_tests {
                 assert!(a.is_normalized() && n.dot(a).abs() < eps);
                 assert!(b.is_normalized() && n.dot(b).abs() < eps);
             }
+        });
+    };
+}
+
+macro_rules! impl_vec3_scalar_shift_op_test {
+    ($vec3:ident, $t_min:literal, $t_max:literal, $rhs_min:literal, $rhs_max:literal) => {
+        glam_test!(test_vec3_scalar_shift_ops, {
+            for x in $t_min..$t_max {
+                for y in $t_min..$t_max {
+                    for z in $t_min..$t_max {
+                        for rhs in $rhs_min..$rhs_max {
+                            assert_eq!(
+                                $vec3::new(x, y, z) << rhs,
+                                $vec3::new(x << rhs, y << rhs, z << rhs)
+                            );
+                            assert_eq!(
+                                $vec3::new(x, y, z) >> rhs,
+                                $vec3::new(x >> rhs, y >> rhs, z >> rhs)
+                            );
+                        }
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec3_scalar_shift_op_tests {
+    ($vec3:ident, $t_min:literal, $t_max:literal) => {
+        mod shift_by_i8 {
+            use glam::$vec3;
+            impl_vec3_scalar_shift_op_test!($vec3, $t_min, $t_max, 0i8, 2);
         }
+        mod shift_by_i16 {
+            use glam::$vec3;
+            impl_vec3_scalar_shift_op_test!($vec3, $t_min, $t_max, 0i16, 2);
+        }
+        mod shift_by_i32 {
+            use glam::$vec3;
+            impl_vec3_scalar_shift_op_test!($vec3, $t_min, $t_max, 0i32, 2);
+        }
+        mod shift_by_u8 {
+            use glam::$vec3;
+            impl_vec3_scalar_shift_op_test!($vec3, $t_min, $t_max, 0u8, 2);
+        }
+        mod shift_by_u16 {
+            use glam::$vec3;
+            impl_vec3_scalar_shift_op_test!($vec3, $t_min, $t_max, 0u16, 2);
+        }
+        mod shift_by_u32 {
+            use glam::$vec3;
+            impl_vec3_scalar_shift_op_test!($vec3, $t_min, $t_max, 0u32, 2);
+        }
+    };
+}
+
+macro_rules! impl_vec3_shift_op_test {
+    ($vec3:ident, $rhs:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec3_shift_ops, {
+            for x1 in $t_min..$t_max {
+                for y1 in $t_min..$t_max {
+                    for z1 in $t_min..$t_max {
+                        for x2 in $t_min..$t_max {
+                            for y2 in $t_min..$t_max {
+                                for z2 in $t_min..$t_max {
+                                    assert_eq!(
+                                        $vec3::new(x1, y1, z1) << $rhs::new(x2, y2, z2),
+                                        $vec3::new(x1 << x2, y1 << y2, z1 << z2)
+                                    );
+                                    assert_eq!(
+                                        $vec3::new(x1, y1, z1) >> $rhs::new(x2, y2, z2),
+                                        $vec3::new(x1 >> x2, y1 >> y2, z1 >> z2)
+                                    );
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec3_shift_op_tests {
+    ($vec3:ident) => {
+        mod shift_ivec3_by_ivec3 {
+            use super::*;
+            impl_vec3_shift_op_test!($vec3, IVec3, 0, 2);
+        }
+        mod shift_ivec3_by_uvec3 {
+            use super::*;
+            impl_vec3_shift_op_test!($vec3, UVec3, 0, 2);
+        }
+    };
+}
+
+macro_rules! impl_vec3_scalar_bit_op_tests {
+    ($vec3:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec3_scalar_bit_ops, {
+            for x in $t_min..$t_max {
+                for y in $t_min..$t_max {
+                    for z in $t_min..$t_max {
+                        for rhs in $t_min..$t_max {
+                            assert_eq!(
+                                $vec3::new(x, y, z) & rhs,
+                                $vec3::new(x & rhs, y & rhs, z & rhs)
+                            );
+                            assert_eq!(
+                                $vec3::new(x, y, z) | rhs,
+                                $vec3::new(x | rhs, y | rhs, z | rhs)
+                            );
+                            assert_eq!(
+                                $vec3::new(x, y, z) ^ rhs,
+                                $vec3::new(x ^ rhs, y ^ rhs, z ^ rhs)
+                            );
+                        }
+                    }
+                }
+            }
+        });
+    };
+}
+
+macro_rules! impl_vec3_bit_op_tests {
+    ($vec3:ident, $t_min:literal, $t_max:literal) => {
+        glam_test!(test_vec3_bit_ops, {
+            for x1 in $t_min..$t_max {
+                for y1 in $t_min..$t_max {
+                    for z1 in $t_min..$t_max {
+                        assert_eq!(!$vec3::new(x1, y1, z1), $vec3::new(!x1, !y1, !z1));
+
+                        for x2 in $t_min..$t_max {
+                            for y2 in $t_min..$t_max {
+                                for z2 in $t_min..$t_max {
+                                    assert_eq!(
+                                        $vec3::new(x1, y1, z1) & $vec3::new(x2, y2, z2),
+                                        $vec3::new(x1 & x2, y1 & y2, z1 & z2)
+                                    );
+                                    assert_eq!(
+                                        $vec3::new(x1, y1, z1) | $vec3::new(x2, y2, z2),
+                                        $vec3::new(x1 | x2, y1 | y2, z1 | z2)
+                                    );
+                                    assert_eq!(
+                                        $vec3::new(x1, y1, z1) ^ $vec3::new(x2, y2, z2),
+                                        $vec3::new(x1 ^ x2, y1 ^ y2, z1 ^ z2)
+                                    );
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        });
     };
 }
 
 mod vec3 {
     use glam::{const_vec3, vec3, BVec3, Vec3};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(12, mem::size_of::<Vec3>());
         assert_eq!(4, mem::align_of::<Vec3>());
         assert_eq!(3, mem::size_of::<BVec3>());
         assert_eq!(1, mem::align_of::<BVec3>());
-    }
+    });
 
-    #[test]
-    fn test_as() {
+    glam_test!(test_as, {
         use glam::{DVec3, IVec3, UVec3, Vec3A};
         assert_eq!(
             DVec3::new(-1.0, -2.0, -3.0),
-            Vec3::new(-1.0, -2.0, -3.0).as_f64()
+            Vec3::new(-1.0, -2.0, -3.0).as_dvec3()
         );
-        assert_eq!(IVec3::new(-1, -2, -3), Vec3::new(-1.0, -2.0, -3.0).as_i32());
-        assert_eq!(UVec3::new(1, 2, 3), Vec3::new(1.0, 2.0, 3.0).as_u32());
+        assert_eq!(
+            IVec3::new(-1, -2, -3),
+            Vec3::new(-1.0, -2.0, -3.0).as_ivec3()
+        );
+        assert_eq!(UVec3::new(1, 2, 3), Vec3::new(1.0, 2.0, 3.0).as_uvec3());
 
         assert_eq!(
             DVec3::new(-1.0, -2.0, -3.0),
-            Vec3A::new(-1.0, -2.0, -3.0).as_f64()
+            Vec3A::new(-1.0, -2.0, -3.0).as_dvec3()
         );
         assert_eq!(
             IVec3::new(-1, -2, -3),
-            Vec3A::new(-1.0, -2.0, -3.0).as_i32()
+            Vec3A::new(-1.0, -2.0, -3.0).as_ivec3()
         );
-        assert_eq!(UVec3::new(1, 2, 3), Vec3A::new(1.0, 2.0, 3.0).as_u32());
+        assert_eq!(UVec3::new(1, 2, 3), Vec3A::new(1.0, 2.0, 3.0).as_uvec3());
 
         assert_eq!(
             IVec3::new(-1, -2, -3),
-            DVec3::new(-1.0, -2.0, -3.0).as_i32()
+            DVec3::new(-1.0, -2.0, -3.0).as_ivec3()
         );
-        assert_eq!(UVec3::new(1, 2, 3), DVec3::new(1.0, 2.0, 3.0).as_u32());
+        assert_eq!(UVec3::new(1, 2, 3), DVec3::new(1.0, 2.0, 3.0).as_uvec3());
         assert_eq!(
             Vec3::new(-1.0, -2.0, -3.0),
-            DVec3::new(-1.0, -2.0, -3.0).as_f32()
+            DVec3::new(-1.0, -2.0, -3.0).as_vec3()
+        );
+        assert_eq!(
+            Vec3A::new(-1.0, -2.0, -3.0),
+            DVec3::new(-1.0, -2.0, -3.0).as_vec3a()
         );
 
         assert_eq!(
             DVec3::new(-1.0, -2.0, -3.0),
-            IVec3::new(-1, -2, -3).as_f64()
+            IVec3::new(-1, -2, -3).as_dvec3()
+        );
+        assert_eq!(UVec3::new(1, 2, 3), IVec3::new(1, 2, 3).as_uvec3());
+        assert_eq!(
+            Vec3::new(-1.0, -2.0, -3.0),
+            IVec3::new(-1, -2, -3).as_vec3()
+        );
+        assert_eq!(
+            Vec3A::new(-1.0, -2.0, -3.0),
+            IVec3::new(-1, -2, -3).as_vec3a()
         );
-        assert_eq!(UVec3::new(1, 2, 3), IVec3::new(1, 2, 3).as_u32());
-        assert_eq!(Vec3::new(-1.0, -2.0, -3.0), IVec3::new(-1, -2, -3).as_f32());
 
-        assert_eq!(DVec3::new(1.0, 2.0, 3.0), UVec3::new(1, 2, 3).as_f64());
-        assert_eq!(IVec3::new(1, 2, 3), UVec3::new(1, 2, 3).as_i32());
-        assert_eq!(Vec3::new(1.0, 2.0, 3.0), UVec3::new(1, 2, 3).as_f32());
-    }
+        assert_eq!(DVec3::new(1.0, 2.0, 3.0), UVec3::new(1, 2, 3).as_dvec3());
+        assert_eq!(IVec3::new(1, 2, 3), UVec3::new(1, 2, 3).as_ivec3());
+        assert_eq!(Vec3::new(1.0, 2.0, 3.0), UVec3::new(1, 2, 3).as_vec3());
+        assert_eq!(Vec3A::new(1.0, 2.0, 3.0), UVec3::new(1, 2, 3).as_vec3a());
+    });
 
     impl_vec3_float_tests!(f32, const_vec3, vec3, Vec3, BVec3);
 }
@@ -838,12 +966,14 @@ mod vec3 {
 mod vec3a {
     use glam::{const_vec3a, vec3a, BVec3A, Vec3A, Vec4};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(16, mem::size_of::<Vec3A>());
         assert_eq!(16, mem::align_of::<Vec3A>());
-        if cfg!(all(target_feature = "sse2", not(feature = "scalar-math"))) {
+        if cfg!(all(
+            any(target_feature = "sse2", target_feature = "simd128"),
+            not(feature = "scalar-math")
+        )) {
             assert_eq!(16, mem::size_of::<BVec3A>());
             assert_eq!(16, mem::align_of::<BVec3A>());
         } else {
@@ -851,10 +981,9 @@ mod vec3a {
             assert_eq!(3, mem::size_of::<BVec3A>());
             assert_eq!(1, mem::align_of::<BVec3A>());
         }
-    }
+    });
 
-    #[test]
-    fn test_mask_align16() {
+    glam_test!(test_mask_align16, {
         // make sure the unused 'w' value doesn't break Vec3Ab behaviour
         let a = Vec4::ZERO;
         let mut b = Vec3A::from(a);
@@ -863,7 +992,7 @@ mod vec3a {
         b.z = 1.0;
         assert!(!b.cmpeq(Vec3A::ZERO).any());
         assert!(b.cmpeq(Vec3A::splat(1.0)).all());
-    }
+    });
 
     #[cfg(all(target_feature = "sse2", not(feature = "scalar-math")))]
     #[test]
@@ -898,14 +1027,13 @@ mod vec3a {
         assert_eq!([0xffffffff, 0, 0xffffffff], a0.0);
     }
 
-    #[test]
-    fn test_min_max_from_vec4() {
+    glam_test!(test_min_max_from_vec4, {
         // checks that the 4th element is unused.
         let v1 = Vec3A::from(Vec4::new(1.0, 2.0, 3.0, 4.0));
         assert_eq!(v1.max_element(), 3.0);
         let v2 = Vec3A::from(Vec4::new(4.0, 3.0, 2.0, 1.0));
         assert_eq!(v2.min_element(), 2.0);
-    }
+    });
 
     impl_vec3_float_tests!(f32, const_vec3a, vec3a, Vec3A, BVec3A);
 }
@@ -913,46 +1041,55 @@ mod vec3a {
 mod dvec3 {
     use glam::{const_dvec3, dvec3, BVec3, DVec3};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(24, mem::size_of::<DVec3>());
-        assert_eq!(8, mem::align_of::<DVec3>());
+        assert_eq!(mem::align_of::<f64>(), mem::align_of::<DVec3>());
         assert_eq!(3, mem::size_of::<BVec3>());
         assert_eq!(1, mem::align_of::<BVec3>());
-    }
+    });
 
     impl_vec3_float_tests!(f64, const_dvec3, dvec3, DVec3, BVec3);
 }
 
 mod ivec3 {
-    use glam::{const_ivec3, ivec3, BVec3, IVec3};
+    use glam::{const_ivec3, ivec3, BVec3, IVec3, UVec3};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(12, mem::size_of::<IVec3>());
         assert_eq!(4, mem::align_of::<IVec3>());
         assert_eq!(3, mem::size_of::<BVec3>());
         assert_eq!(1, mem::align_of::<BVec3>());
-    }
+    });
 
     impl_vec3_signed_tests!(i32, const_ivec3, ivec3, IVec3, BVec3);
     impl_vec3_eq_hash_tests!(i32, ivec3);
+
+    impl_vec3_scalar_shift_op_tests!(IVec3, -2, 2);
+    impl_vec3_shift_op_tests!(IVec3);
+
+    impl_vec3_scalar_bit_op_tests!(IVec3, -2, 2);
+    impl_vec3_bit_op_tests!(IVec3, -2, 2);
 }
 
 mod uvec3 {
-    use glam::{const_uvec3, uvec3, BVec3, UVec3};
+    use glam::{const_uvec3, uvec3, BVec3, IVec3, UVec3};
 
-    #[test]
-    fn test_align() {
+    glam_test!(test_align, {
         use std::mem;
         assert_eq!(12, mem::size_of::<UVec3>());
         assert_eq!(4, mem::align_of::<UVec3>());
         assert_eq!(3, mem::size_of::<BVec3>());
         assert_eq!(1, mem::align_of::<BVec3>());
-    }
+    });
 
     impl_vec3_tests!(u32, const_uvec3, uvec3, UVec3, BVec3);
     impl_vec3_eq_hash_tests!(u32, uvec3);
+
+    impl_vec3_scalar_shift_op_tests!(UVec3, 0, 2);
+    impl_vec3_shift_op_tests!(UVec3);
+
+    impl_vec3_scalar_bit_op_tests!(UVec3, 0, 2);
+    impl_vec3_bit_op_tests!(UVec3, 0, 2);
 }
